/*
 * Kneron Example CPU Operation driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
 
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "base.h"
#include "kdpio.h"
#include "ipc.h"
#include "cpu_ops_ex.h"
#include "dbg.h"

inline static int pad_up_16(int a)
{
    return ceil((float)a / 16) * 16;
}

int nearest_upsample_cpu(int cpu_op, struct kdp_image_s *image_p)
{
    int row_num_in = CPU_OP_NODE_INPUT_ROW(image_p);
    int col_num_in = CPU_OP_NODE_INPUT_COL(image_p);
    int chnl_num_in = CPU_OP_NODE_INPUT_CH(image_p);

    int row_num_out = CPU_OP_NODE_OUTPUT_ROW(image_p);
    int col_num_out = CPU_OP_NODE_OUTPUT_COL(image_p);

    int col_num_pad_in = pad_up_16(col_num_in);
    int col_num_pad_out = pad_up_16(col_num_out);
    int col_num_out_32 = col_num_out >> 2;

    int oi, oj, ij, k;

    int h_scale = row_num_out / row_num_in;
    int w_scale = col_num_out / col_num_in;
    int out_index, in_index, in_base;

    uint8_t *data_array = (uint8_t*)CPU_OP_NODE_INPUT_ADDR(image_p);
    uint8_t *dst_data_array = (uint8_t*)CPU_OP_NODE_OUTPUT_ADDR(image_p);
    uint8_t in_temp[4];
    uint8_t out_temp[4];
    uint8_t in_index_array[col_num_pad_out];
    int temp;
    uint8_t mod = 0;

    for (oi = 0; oi < col_num_pad_out; oi++) {
        in_index_array[oi] = oi / w_scale;
    }

    //HCW for DRAM
    for (oj = 0; oj < row_num_out; oj++) {
        ij = oj / h_scale;
        for (k = 0; k < chnl_num_in; ++k) {
            out_index = col_num_pad_out * (k + chnl_num_in * oj);
            in_base = col_num_pad_in * (k + chnl_num_in * ij);
            for (oi = 0; oi <= col_num_out_32; oi++) {
                temp = oi << 2;
                in_index = in_index_array[temp] + in_base;
                *((uint32_t*)in_temp) = *((uint32_t*)(data_array + in_index));

                out_temp[0] = in_temp[0];
                out_temp[1] = in_temp[in_index_array[temp + 1] - in_index_array[temp]];
                out_temp[2] = in_temp[in_index_array[temp + 2] - in_index_array[temp]];
                out_temp[3] = in_temp[in_index_array[temp + 3] - in_index_array[temp]];

                if (oi != col_num_out_32) {
                    *((uint32_t*)(dst_data_array + out_index) + oi) = *((uint32_t*)out_temp);
                }
                else {
                    uint8_t i = 0;
                    mod = (col_num_out & 0x3);
                    for (i = 0; i < mod; i++) {
                        *((uint8_t*)((uint32_t*)(dst_data_array + out_index) + oi) + i) = out_temp[i];
                    }
                }
            }
        }
    }

    return 0;
}

int zero_upsample_cpu(int cpu_op, struct kdp_image_s *image_p) 
{
    int row_num_in = CPU_OP_NODE_INPUT_ROW(image_p);
    int col_num_in = CPU_OP_NODE_INPUT_COL(image_p);
    int chnl_num_in = CPU_OP_NODE_INPUT_CH(image_p);

    int row_num_out = CPU_OP_NODE_OUTPUT_ROW(image_p);
    int col_num_out = CPU_OP_NODE_OUTPUT_COL(image_p);

    int col_num_pad_in = pad_up_16(col_num_in);
    int col_num_pad_out = pad_up_16(col_num_out);
    int col_num_out_32 = col_num_out >> 2;

    int oi, oj, ij, k;

    int h_scale = (row_num_out - 1) / (row_num_in - 1);
    int w_scale = (col_num_out - 1) / (col_num_in - 1);
    int out_index, in_index, in_base;

    uint8_t *data_array = (uint8_t*)CPU_OP_NODE_INPUT_ADDR(image_p);
    uint8_t *dst_data_array = (uint8_t*)CPU_OP_NODE_OUTPUT_ADDR(image_p);
    memset(dst_data_array, 0, col_num_pad_out * row_num_out * chnl_num_in);
    uint8_t in_temp[4];
    uint8_t out_temp[4] = {0, 0, 0, 0};
    uint8_t in_index_array[col_num_pad_out];
    int temp;
    for (oi = 0; oi < col_num_pad_out; oi++) {
        in_index_array[oi] = oi / w_scale;
    }

    //HCW for DRAM
    for (oj = 0; oj < row_num_out; oj++) {
        if (oj % 2 != 0) continue;
        ij = oj / h_scale;
        for (k = 0; k < chnl_num_in; ++k) {
            out_index = col_num_pad_out * (k + chnl_num_in * oj);
            in_base = col_num_pad_in * (k + chnl_num_in * ij);

            for (oi = 0; oi <= col_num_out_32; oi++) {
                temp = oi << 2;
                in_index = in_index_array[temp] + in_base;
                *((uint32_t*)in_temp) = *((uint32_t*)(data_array + in_index));

                out_temp[0] = in_temp[0];
                out_temp[2] = in_temp[in_index_array[temp + 2] - in_index_array[temp]];

                *((uint32_t*)(dst_data_array + out_index) + oi) = *((uint32_t*)out_temp);
            }
        }
    }
    return 0;
}
