/*
 * KDP command adapter header
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#ifndef __KDP_COMMAND_ADAPTER_H__
#define __KDP_COMMAND_ADAPTER_H__

#include <cmsis_os2.h>
#include "kmdw_camera.h"
#include "base.h"
#include "framework/framework_driver.h"

/* Configuration */
#define	    CMD_NIR_RGB_EN			            (1)


#define     UART_RX_BUFFER_SIZE                (4196)
#define     UART_TX_BUFFER_SIZE                200//3000//(4096)

/* Packet header definition */
#define		CMD_HEAD_RX                         (0x78875AA5)
#define		CMD_HEAD_RX_1						((CMD_HEAD_RX&0xFF000000)>>24 )
#define		CMD_HEAD_RX_2						((CMD_HEAD_RX&0x00FF0000)>>16 )
#define		CMD_HEAD_RX_3						((CMD_HEAD_RX&0x0000FF00)>>8 )
#define		CMD_HEAD_RX_4						((CMD_HEAD_RX&0x000000FF)>>0 )

#define		CMD_HEAD_TX						    (~(0x78875AA5))
#define		CMD_TAIL							(0x7887)
#define		CMD_TAIL_1						    (CMD_TAIL&0xFF)
#define		CMD_TAIL_2						    ((CMD_TAIL>>8)&0xFF)


//error code
#define		CMD_PACKET_HEAD_CHECK_ERROR		    (0xE0)
#define		CMD_PACKET_TAIL_CHECK_ERROR			(0xE1)
#define		CMD_PACKET_OK						(0x66)


//cmd code
#define		CMD_PROGRAM_ACT					    (0xF0)
#define		CMD_READ_ACT						(0xF1)
#define		CMD_ERASE_SECTOR_ACT				(0xF2)
#define		CMD_ACT						        (0x00)		//for future use
#define		CMD_READ_NIR_ACT					(0xE1)
#define		CMD_READ_RGB_ACT					(0xE2)

#define		CMD_ACT_NUM_INIT				    (0x01)
#define		CMD_ACT_NUM_NIR				        (0x02)
#define		CMD_ACT_NUM_RGB				        (0x03)
#define		CMD_ACT_RESERVED		            (0x04)
#define		CMD_ACT_NUM_SW_VERSION		        (0x05)


//status code
#define		CMD_STATUS_OK				        (0x7855)
#define		CMD_STATUS_FAIL			            (0xEEEE)


//---- command machine status
#define		CMD_DRV_FAIL		                (0xE2)
#define		CMD_DRV_OK		                    (1)
#define		CMD_DRV_BUSY		                (2)

enum cmd_parser_flow
{
	cmd_parser_init = 0x00,
	cmd_parser_flash_program = 0xF0,
	cmd_parser_flash_read = 0xF1,
	cmd_parser_flash_erase = 0xF2,
	cmd_parser_NIR = 0xE1,
	cmd_parser_RGB = 0xE2,
	cmd_parser_idle = 0xFF ,
};

enum e_cmd_flow
{
	cmd_init = 0,				//clear Uart buffer and flags!
	cmd_packet_analyze,
	cmd_response_doing,
	cmd_error_report,
	cmd_idle,
};

struct s_cmd_format
{
	uint32_t	head;					//packet start
	uint16_t	host_number;
	uint16_t	cmd_stat;
	uint16_t	sector_index;
	uint8_t 	action_number;
	uint16_t	data_size;			    //how many bytes need to read or program
	uint32_t	check_sum;
	uint16_t	tail;					//packet end

	uint16_t	status;					//response use

	//packet related
	uint8_t		busy_flag;
	uint8_t		*ptr;
	uint32_t	*ptr_index;
	uint16_t	data_offset;
	uint32_t	buf_max_size;
	//uint16_t	data_end_offset;
	uint32_t	ddr_ptr;
};

struct kdp_cmd_adapter
{
	enum e_cmd_flow flow;
	struct s_cmd_format 	receive_cmd;
	struct s_cmd_format 	send_response;
	uint32_t	*temp_buffer;
	uint32_t	temp_buffer_index;
	uint32_t	temp_buffer_size;
};

void kdp_cmd_adapter_init(void);

extern uint8_t		g_uart_Rx_buffer[];
extern uint32_t		g_uart_Rx_buffer_index;


#endif // __KDP_COMMAND_ADAPTER_H__

