/********************************************************************
 * Copyright (c) 2020 Kneron, Inc. All Rights Reserved.
 *
 * The information contained herein is property of Kneron, Inc.
 * Terms and conditions of usage are described in detail in Kneron
 * STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information.
 * NO WARRANTY of ANY KIND is provided. This heading must NOT be removed
 * from the file.
 ********************************************************************/
     
/**@addtogroup  KDP_INFERENCE  KDP_INFERENCE
 * @{
 * @brief       Header file for kdp inference
 * @copyright   Copyright (C) 2020 Kneron, Inc. All rights reserved.
 */
#ifndef __KDP_INFERENCE_H__
#define __KDP_INFERENCE_H__

#include "board_kl520.h"
#include "base.h"
#include "kmdw_camera.h"
#include "kmdw_display.h"
#include "framework/result.h"

/**
 * @defgroup KDP_INFERENCE_EVT_FLAG KDP_INFERENCE_EVT_FLAG
 * @brief    kdp inference event flag
 * @{ */
#define FLAGS_KDP520_OPEN_CAMERA0_EVT               BIT0        /**< 1 << 0*/
#define FLAGS_KDP520_START_CAMERA0_EVT              BIT1        /**< 1 << 1*/
#define FLAGS_KDP520_STOP_CAMERA0_EVT               BIT2        /**< 1 << 2*/
#define FLAGS_KDP520_CLOSE_CAMERA0_EVT              BIT3        /**< 1 << 3*/
#define FLAGS_KDP520_OPEN_CAMERA1_EVT               BIT4        /**< 1 << 4*/
#define FLAGS_KDP520_START_CAMERA1_EVT              BIT5        /**< 1 << 5*/
#define FLAGS_KDP520_STOP_CAMERA1_EVT               BIT6        /**< 1 << 6*/
#define FLAGS_KDP520_CLOSE_CAMERA1_EVT              BIT7        /**< 1 << 7*/
#define FLAGS_KDP520_RESERVED_1                     BIT8        /**< Reserved 1 << 8*/
#define FLAGS_KDP520_RESERVED_2                     BIT9        /**< Reserved 1 << 9*/
#define FLAGS_KDP520_START_INFERENCE_LOOP_EVT       BIT10       /**< 1 << 10*/
#define FLAGS_KDP520_STOP_INFERENCE_LOOP_EVT        BIT11       /**< 1 << 11*/
#define FLAGS_KDP520_START_INFERENCE_EVT            BIT12       /**< 1 << 12*/
#define FLAGS_KDP520_REGISTER_FDR_EVT               BIT13       /**< 1 << 13*/
#define FLAGS_KDP520_AUTO_REGISTER_FDR_EVT          BIT14       /**< 1 << 14*/
#define FLAGS_KDP520_REGISTER_CFM_FDR_EVT           BIT15       /**< 1 << 15*/
#define FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT     BIT16       /**< 1 << 16*/
#define FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT      BIT17       /**< 1 << 17*/
#define FLAGS_KDP520_AUTO_REGISTER_5_FACE_EVT       BIT18       /**< 1 << 18*/
#define FLAGS_KDP520_CANCEL_REGISTER_FDR_EVT        BIT19       /**< 1 << 19*/
#define FLAGS_KDP520_ADD_FDR_FACE_EVT               BIT20       /**< 1 << 20*/
#define FLAGS_KDP520_DEL_FDR_FACE_EVT               BIT21       /**< 1 << 21*/
#define FLAGS_KDP520_CHANGE_DRAW_TIMEOUT_EVT        BIT22       /**< 1 << 22*/
#define FLAGS_KDP520_RESERVED_4                     BIT23       /**< Reserved 1 << 23*/
/** @}*/

#define FLAGS_KDP520_ALL_EVENTS    ( FLAGS_KDP520_OPEN_CAMERA0_EVT | \
                                    FLAGS_KDP520_START_CAMERA0_EVT | \
                                    FLAGS_KDP520_STOP_CAMERA0_EVT | \
                                    FLAGS_KDP520_CLOSE_CAMERA0_EVT | \
                                    FLAGS_KDP520_OPEN_CAMERA1_EVT | \
                                    FLAGS_KDP520_START_CAMERA1_EVT | \
                                    FLAGS_KDP520_STOP_CAMERA1_EVT | \
                                    FLAGS_KDP520_CLOSE_CAMERA1_EVT | \
                                    FLAGS_KDP520_START_INFERENCE_LOOP_EVT | \
                                    FLAGS_KDP520_STOP_INFERENCE_LOOP_EVT | \
                                    FLAGS_KDP520_START_INFERENCE_EVT | \
                                    FLAGS_KDP520_REGISTER_FDR_EVT | \
                                    FLAGS_KDP520_AUTO_REGISTER_FDR_EVT | \
                                    FLAGS_KDP520_REGISTER_CFM_FDR_EVT | \
                                    FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT | \
                                    FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT | \
                                    FLAGS_KDP520_AUTO_REGISTER_5_FACE_EVT | \
                                    FLAGS_KDP520_CANCEL_REGISTER_FDR_EVT | \
                                    FLAGS_KDP520_ADD_FDR_FACE_EVT | \
                                    FLAGS_KDP520_DEL_FDR_FACE_EVT)


#define DB_SLOT_SIZE                   (MAX_USER)


/**
* Structure to represent camera content which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_camera_ctx {
    bool init;
    enum camera_state state;    /**< Camera state, see @ref camera_state*/
    uint32_t width;             /**< Camera image width*/
    uint32_t height;            /**< Camera image hieght*/
    uint32_t pixelformat;       /**< Camera image pixel format*/
} kdp_camera_ctx;


/*----------------------------------------------------------------------------------
 *    Communication struct parameters between Application <-> Moddleware
 *----------------------------------------------------------------------------------*/
/**
* Structure to represent face register content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_rgst_context {
    osThreadId_t caller_tid;    /**< Caller thread id */
    int rgst_type;              /**< see @ref FLAGS_KDP520_REGISTER_FDR_EVT ~ @ref FLAGS_KDP520_REGISTER_CFM_FDR_EVT */
    int user_idx;               /**< User id to be registered*/
    int rgst_idx;               /**< Only for auto register usage*/
    int num_rgst;               /**< Number of face register */ 
    int ret;                    /**< Return the fdfr register result*/
};

/**
* Structure to represent 5 face register content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_rgst_5_context {
    osThreadId_t caller_tid;    /**< Caller thread id */
    int user_idx;               /**< User id to be registered*/
    int pose_type;
    int ret;                    /**< Return the fdfr register result*/
};

/**
* Structure to represent face add content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_add_context {
    osThreadId_t caller_tid;    /**< Caller thread id */
    int user_idx;               /**< User id to be added */
    int ret;                    /**< Return the fdfr add result*/
};

/**
* Structure to represent face delete content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_del_context {
    osThreadId_t caller_tid;    /**< Caller thread id */
    int user_idx;               /**< User id to be delete */
    bool all;                   /**< Delete all user dababase or delete the specific user datace */
    int ret;                    /**< Return the fdfr delete result*/
};

/**
* Structure to represent minimum requirment content which be commnuicated between Application <-> Moddleware
*/
struct kdp_fdfr_context {
    osThreadId_t caller_tid;    /**< Caller thread id */
    int ret;                    /**< Return the fdfr result*/
};

/**
* typedef structure to represent camera commnucation parameter which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_camera_comm_params {
    osThreadId_t caller_tid;    /**< Caller thread id */
    int cam_idx;                /**< Camera source index, see @ref Camera_setting*/
    int ret;                    /**< Return the camera setting result*/
} kdp_camera_comm_params;

/**
* typedef structure to represent fdfr commnucation parameter which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_fdfr_comm_params {
    union {
        struct kdp_fdfr_context         fdfr_ctx;           /**< see @ref kdp_fdfr_context */
        struct kdp_fdfr_rgst_context    rgst_ctx;           /**< see @ref kdp_fdfr_rgst_context */
        struct kdp_fdfr_rgst_5_context  rgst_5_ctx;         /**< see @ref kdp_fdfr_rgst_5_context */
        struct kdp_fdfr_context         rgst_cancel_ctx;    /**< see @ref kdp_fdfr_context */
        struct kdp_fdfr_add_context     add_ctx;            /**< see @ref kdp_fdfr_add_context */
        struct kdp_fdfr_del_context     del_ctx;            /**< see @ref kdp_fdfr_del_context */
    };
} kdp_fdfr_comm_params;

/**
* typedef structure to represent all commnucation parameter which be commnuicated between Application <-> Moddleware
*/
typedef struct kdp_all_comm_params {
    union {
        kdp_camera_comm_params cam_params;      /**< see @ref kdp_camera_comm_params */
        kdp_fdfr_comm_params   fdfr_params;     /**< see @ref kdp_fdfr_comm_params */
    };
} kdp_all_comm_params;


/**
* typedef structure to represent kdp fdfr FSM table
*/
typedef struct kdp_fsm_table {
    int state;                                                      /**< FSM */
    int evt;                                                        /**< Event flag, see @ref KDP_INFERENCE_EVT_FLAG */
    void (*evtActFunc)(int evt, kdp_fdfr_comm_params *fdfr_params); /**< Action function */
    int next_state;                                                 /**< Next FSM */
} kdp_fsm_table;

/**
* typedef structure to represent kdp fdfr FSM control
*/
typedef struct kdp_fsm {
    kdp_fsm_table *pfsm_table;  /**< see @ref kdp_fsm_table */
    int curr_state;             /**< Current FSM */
} kdp_fsm;

/** Enumerations of the count of kdp fdfr inference source */
enum kdp_inference_imgsrc {
    KDP_INFERENCE_SINGLE_IMGSRC = 0,    /**< Single image source for KDP inference*/
    KDP_INFERENCE_BOTH_IMGSRC           /**< Both image sources for KDP inference*/
};

/** Enumerations of kdp fdr FSM */
enum kdp_fdr_fsm {
    KDP_FDR_FSM_UNINIT = 0,         /**< KDP fdr inference thread isn't created yet*/
    KDP_FDR_FSM_IDLE,               /**< KDP fdr inference thread is already created and wait for event*/
    KDP_FDR_FSM_INFERENCE,          /**< KDP fdr inference thread is running fdfr inference*/
    KDP_FDR_FSM_REGISTER,           /**< KDP fdr inference thread is running fdfr register*/
    KDP_FDR_FSM_CANCEL_REGISTER,    /**< KDP fdr inference thread is running fdfr abort register*/
    KDP_FDR_FSM_ADD,                /**< KDP fdr inference thread is running to add fdfr register result*/
    KDP_FDR_FSM_DELETE              /**< KDP fdr inference thread is running to delete fdfr record*/
};

extern kdp_camera_ctx m_kdp_cam_ctx[IMGSRC_NUM + 2];
extern kdp_all_comm_params m_kdp_all_comm_params;
extern osThreadId_t tid_inference;
extern int reg_type;
extern int reg_count;
extern int m_face_add_timeout;
extern int m_face_recognition_timeout;
extern uint32_t m_fdfr_tick;
extern enum kdp_inference_imgsrc m_inf_imgsrc;
/*-----------------------------------------------------------------------
 *                          Puclic FDFR API 
 *-----------------------------------------------------------------------*/
/**
* @brief        Initialize fdfr model
*
* @param[in]    N/A
* @return       TRUE for success, or FALSE for error
*/
bool kdp_fdfr_model_init(void);

/**
* @brief        Get the inference result
*
* @param[in]    N/A
* @return       see @ref fdr_result
*/
fdr_result kdp_fdfr_get_inference_result(void);

/**
* @brief        Set fps of draw thread
*
* @note         fsp range 5 ~ 30
*
* @param[in]    fps  frame per second
* @return       0 for success, or -1 for error
*/
void kdp_fdfr_set_draw_fps(int fps);

/**
* @brief        Create draw thread to handle draw task
*
* @detail       This thread will wait for @FLAGS_KDP520_CHANGE_DRAW_TIMEOUT_EVT to start loop drawing fdr result\n
*               While this thread starts loop drawing, the fps is 10 by default.\n
*               You can call @kdp_fdfr_set_draw_fps to change the draw fps
*
* @param[in]    N/A
* @return       0 for success, or -1 for error
*/
int kdp_fdfr_draw_thread_create(void);
    
/**
* @brief        Create fdfr thread to handle inference task
*
* @param[in]    N/A
* @return       0 for success, or -1 for error
*/
int kdp_fdfr_inference_thread_create(void);

/**
* @brief        Set the fdfr boundingbox represetation method
*
* @param[in]    mode     Draw fdfr boundingbox mode, see @ref KDP_BOUNDINGBOX_MODE
* @return       N/A
*/
void kdp_fdfr_set_boundingbox_mode(int mode);

/**
* @brief        Get started to do fdfr inference
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_fdfr_inference_start(void);

/**
* @brief        Stop to do fdfr inference
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_fdfr_inference_stop(void);

/**
* @brief        Check whether fdfr inference is running
*
* @param[in]    N/A
* @return       TRUE for YES, or FALSE for NOT
*/
bool kdp_fdfr_inference_is_running(void);

/**
* @brief        Get the avaliable database slot, max user @ref DB_SLOT_SIZE
*
* @param[in]    N/A
* @return       Index of the available databse slot
*/
int kdp_fdfr_get_available_db_idx(void);

/**
* @brief        Get the remaining count of fdfr feature map for one user, max feature map @ref MAX_FID
*
* @param[in]    User register index
* @return       The remaining count of fdfr feature map for one user
*/
int kdp_fdfr_get_remaining_fmap_cnt(int user_idx);

/**
* @brief        Reset some parameters for fdfr inference usage
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_fdfr_recognition_variable_reset(void);

/**
* @brief        Do fdfr inference at @ref KDP_FDR_FSM_INFERENCE
*
* @param[in]    evt          Event flag which sent from kl520_api_fdfr
* @param[in]    fdfr_params  Pointer to @ref kdp_fdfr_comm_params
* @return       N/A
*/
void kdp_fdr_do_inference(int evt, kdp_fdfr_comm_params *fdfr_params);

/**
* @brief        Do fdfr register at @ref KDP_FDR_FSM_REGISTER
*
* @param[in]    evt          Event flag which sent from kl520_api_fdfr
* @param[in]    fdfr_params  Pointer to @ref kdp_fdfr_comm_params
* @return       N/A
*/
void kdp_fdr_do_register(int evt, kdp_fdfr_comm_params *fdfr_params);

/**
* @brief        Do fdfr abort register at @ref KDP_FDR_FSM_CANCEL_REGISTER
*
* @param[in]    evt          Event flag which sent from kl520_api_fdfr
* @param[in]    fdfr_params  Pointer to @ref kdp_fdfr_comm_params
* @return       N/A
*/
void kdp_fdr_do_cancel_register(int evt, kdp_fdfr_comm_params *fdfr_params);

/**
* @brief        Do fdfr add at @ref KDP_FDR_FSM_ADD
*
* @param[in]    evt          Event flag which sent from kl520_api_fdfr
* @param[in]    fdfr_params  Pointer to @ref kdp_fdfr_comm_params
* @return       N/A
*/
void kdp_fdr_do_add(int evt, kdp_fdfr_comm_params *fdfr_params);

/**
* @brief        Do fdfr delete at @ref KDP_FDR_FSM_DELETE
*
* @param[in]    evt          Event flag which sent from kl520_api_fdfr
* @param[in]    fdfr_params  Pointer to @ref kdp_fdfr_comm_params
* @return       N/A
*/
void kdp_fdr_do_delete(int evt, kdp_fdfr_comm_params *fdfr_params);

/**
* @brief        Enable snapshot RBG display frame buffer\n
* @details      This function only snapshot one RBG display frame buffer\n
*               The RBG display frame buffer will keep in DDR\n
*               If you want to keep it forever, you need to call kdp_camera_snapshot_save()
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_camera_enable_snapshot_dp(void);

/**
* @brief        Save snapshot RBG display frame buffer from DDR to Flash in sequentail cycle
* @details      Size of ring buffer @ref KDP_FLASH_SNAPSHOT_NUMS
*
* @param[in]    N/A
* @return       index of snapshot record
*/
int kdp_camera_snapshot_save(void);

/**
* @brief        Snapshot RGB display frame buffer to DDR
* @details      This function only works after call @ref kdp_camera_enable_snapshot_dp()\n
*
* @param[in]    addr      Display frame buffer address
* @return       FALSE     @ref kdp_camera_enable_snapshot_dp() isn't called or during snapshot RGB display frame buffer\n
*               TRUE      Successd to snapshot display frame buffer
*/
bool kdp_camera_snapshot(uint32_t addr);

/**
* @brief        Resotre snapshot history by index and show on display preview
*
* @param[in]    idx       Which index of snapshot history will be resotred
* @return       FALSE     Index of snapshot history is over than @ref KDP_FLASH_SNAPSHOT_NUMS\n
*               TRUE      Secceed to resotre snapshot history by index and show on display preview
*/
bool kdp_camera_review_snapshot_history(int idx);

/**
* @brief        Disable snapshot history preview
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_camera_disable_snapshot_review(void);

/**
* @brief        Enable snapshot RBG+NIR inference frame buffer\n
* @details      This function only snapshot one set of RBG+NIR inference frame buffer\n
*               The set of RBG+NIR inference frame buffer will keep in DDR\n
*               If you want to keep it forever, you need to call kdp_fdfr_snapshot_save()
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_camera_enable_snapshot_fdfr(void);

/**
* @brief        Save snapshot a set of RBG+NIR inference frame buffer from DDR to Flash
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_fdfr_snapshot_save(void);

/**
* @brief        Snapshot RGB+NIR inference frame buffer to DDR
* @details      This function only works after call @ref kdp_camera_enable_snapshot_fdfr()\n
*
* @param[in]    cam_idx     Camera srouce index 
*               addr        Address of fdfr inference frame buffer
* @return       FALSE       @ref kdp_camera_enable_snapshot_fdfr() isn't called or camera index is wrong\n
*               TRUE        Secceed to snapshot display frame buffer
*/
bool kdp_fdfr_snapshot(uint8_t cam_idx, uint32_t addr);

/**
* @brief        Resotre a set of RGB+NIR inference frame buffer from FLASH to DDR
*
* @param[in]    N/A
* @return       N/A
*/
void kdp_fdfr_review_snapshot(void);

/**
* @brief        Get the address of snapshot image
*
* @param[in]    cam_idx     Camera srouce index 
* @return       0           Buufer is not ready
*               >0          Address of snapshot image
*/
uint32_t kdp_fdfr_get_snapshot_addr(uint8_t cam_idx);

/*-----------------------------------------------------------------------
 *                          Puclic camera API 
 *-----------------------------------------------------------------------*/
int kdp_camera_setting(uint8_t cam_idx, uint32_t width, uint32_t height, uint32_t pixelformat);

int kdp_get_camera_ctx_state(uint8_t cam_idx);

osThreadId_t kdp_get_inference_thread_id(void);

#endif
/** @}*/
