/*
 * KDP Command Adapter
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
#include "board_kl520.h"
#include <stdio.h> 
#include <stdlib.h>
#include <cmsis_os2.h>
#include "base.h"
#include "RTX_Config.h"
#include "kdp_ddr_table.h"

#include "kl520_api_fdfr.h"     
#include "kl520_include.h"
#include "kdev_flash.h"
#include "media/kdp_command_adapter.h"
#include "media/kdp_inference.h"
#include "kdrv_uart.h"
#include "kmdw_console.h"

osThreadId_t    cmd_adapter_tid = NULL;

uint8_t         g_uart_Rx_buffer[UART_RX_BUFFER_SIZE] __attribute__((aligned (4)));  //put it to DDR
uint32_t        g_uart_Rx_buffer_index = 0;
uint8_t         g_uart_Tx_buffer[UART_TX_BUFFER_SIZE] __attribute__((aligned (4)));
uint32_t        g_uart_Tx_buffer_index = 0;

struct kdp_cmd_adapter   m_cmd_adapter;

void Drv_OTA_init(struct kdp_cmd_adapter *p_cmd_adapter)
{
    p_cmd_adapter->receive_cmd.ptr = g_uart_Rx_buffer; //4KB
    p_cmd_adapter->receive_cmd.ptr_index = &g_uart_Rx_buffer_index;
    *p_cmd_adapter->receive_cmd.ptr_index = 0;

    p_cmd_adapter->send_response.ptr = g_uart_Rx_buffer;//gu8OTA_Tx_buffer;
    p_cmd_adapter->send_response.ptr_index = &g_uart_Tx_buffer_index;
    *p_cmd_adapter->send_response.ptr_index = 0;
}

uint32_t Drv_utility_checksum(uint8_t * buf, uint16_t start, uint16_t end)
{
	uint32_t ntemp = 0;
	uint32_t i;

	for(i = start ; i < end; i++)
	{
		ntemp += *(buf+i);
	}
	return	ntemp;
}

uint8_t Drv_OTA_packet_analyze(struct kdp_cmd_adapter *p_cmd_adapter)
{
	uint32_t	i = 0;
	uint8_t	    *ptr = p_cmd_adapter->receive_cmd.ptr;
	uint32_t	*ptr_indedx = p_cmd_adapter->receive_cmd.ptr_index;
	uint32_t	nhead_index = 0xFFFFFFFF;
	uint32_t	ncheck_sum = 0;

	p_cmd_adapter->receive_cmd.busy_flag = 1;		//flash controller is busy

	//-----------------------
	//packet tail  check
	//-----------------------
	if((*(ptr + (*ptr_indedx) -1) != CMD_TAIL_1) || ( *(ptr + (*ptr_indedx) -2 ) != CMD_TAIL_2))
	{
		return CMD_PACKET_TAIL_CHECK_ERROR;
	}

	//-----------------------
	//-----	head check ------
	//-----------------------
	for(i = 0; i < ( *ptr_indedx-4 ); i++)
	{
		if(	(*(ptr + i + 0) == CMD_HEAD_RX_1) && (*(ptr +i + 1) == CMD_HEAD_RX_2)
			&& (*(ptr +i + 2) == CMD_HEAD_RX_3) && (*(ptr +i + 3) == CMD_HEAD_RX_4))
		{
			nhead_index = i;
			*(ptr + i + 0) = 0;
			*(ptr +i + 1) = 0;
			break;
		}
	}
	if(nhead_index == 0xFFFFFFFF)
	{
		return CMD_PACKET_HEAD_CHECK_ERROR;
	}

	p_cmd_adapter->receive_cmd.head = CMD_HEAD_RX;							//receive data from device
	p_cmd_adapter->receive_cmd.host_number = *(ptr+nhead_index+4)<<8 | *(ptr+nhead_index+5);
	p_cmd_adapter->receive_cmd.cmd_stat = *(ptr+nhead_index+6)<<8 | *(ptr+nhead_index+7);
	p_cmd_adapter->receive_cmd.action_number = 0;

    if(p_cmd_adapter->receive_cmd.cmd_stat == CMD_READ_ACT 
        || p_cmd_adapter->receive_cmd.cmd_stat == CMD_READ_NIR_ACT
            || p_cmd_adapter->receive_cmd.cmd_stat == CMD_READ_RGB_ACT)
	{
    	p_cmd_adapter->receive_cmd.action_number = *(ptr+nhead_index+8);
		p_cmd_adapter->receive_cmd.sector_index = (*( ptr+nhead_index+8) << 8) | (*(ptr+nhead_index + 9));
		p_cmd_adapter->receive_cmd.data_size =(*(ptr+nhead_index+10) << 8) | (*(ptr+nhead_index+11));
		p_cmd_adapter->receive_cmd.data_offset = 0;
		ncheck_sum = Drv_utility_checksum( ptr, nhead_index+6, *ptr_indedx - 6 );
	}
	else if( p_cmd_adapter->receive_cmd.cmd_stat == CMD_PROGRAM_ACT )
	{
		p_cmd_adapter->receive_cmd.sector_index= ( *( ptr+nhead_index+8) << 8 ) | *(ptr+nhead_index + 9);
        DSG("***********[OTA] PROGRAM sector_index = %d..", p_cmd_adapter->receive_cmd.sector_index);
		p_cmd_adapter->receive_cmd.action_number = 0;
		p_cmd_adapter->receive_cmd.data_size=	( *( ptr+nhead_index+10) << 8 ) | *(ptr+nhead_index+11);

		p_cmd_adapter->receive_cmd.data_offset = nhead_index+12;

		ncheck_sum = Drv_utility_checksum( ptr, nhead_index+6, *ptr_indedx - 6 );
	}
	else if( p_cmd_adapter->receive_cmd.cmd_stat == CMD_ERASE_SECTOR_ACT )
	{
		p_cmd_adapter->receive_cmd.sector_index =	( *( ptr+nhead_index+8) << 8 ) | *(ptr+nhead_index + 9);			//start sector
		p_cmd_adapter->receive_cmd.data_offset =	( *( ptr+nhead_index+10) << 8 ) | *(ptr+nhead_index + 11);		//end sector

		p_cmd_adapter->receive_cmd.data_size=	0;
		ncheck_sum = Drv_utility_checksum( ptr, nhead_index+6, *ptr_indedx - 6 );
	}
	else	// if(p_cmd_adapter->receive_cmd.cmd_stat == CMD_ACT)
	{
		p_cmd_adapter->receive_cmd.action_number = *( ptr+nhead_index + 8 );
		//action
		if((p_cmd_adapter->receive_cmd.action_number == CMD_ACT_NUM_NIR) ||
            (p_cmd_adapter->receive_cmd.action_number == CMD_ACT_NUM_RGB))
		{
			kl520_api_enable_snapshot_fdfr();
		}

		p_cmd_adapter->receive_cmd.sector_index = 0;
		p_cmd_adapter->receive_cmd.data_size=	0;

		ncheck_sum = Drv_utility_checksum( ptr, nhead_index+6, *ptr_indedx - 6 );
		p_cmd_adapter->receive_cmd.data_offset = 0;
	}

	p_cmd_adapter->receive_cmd.check_sum = (*( ptr+ (*ptr_indedx) -6 ) << 24 ) | (*(ptr+nhead_index+ (*ptr_indedx) -5 )<<16 ) |
										   (*( ptr+ (*ptr_indedx) -4 ) << 8 ) | *(ptr+nhead_index+ (*ptr_indedx) -3 );

	p_cmd_adapter->receive_cmd.tail = CMD_TAIL;

	//check sum error
	if(p_cmd_adapter->receive_cmd.check_sum != ncheck_sum)
	{
		return CMD_PACKET_HEAD_CHECK_ERROR;
	}
	return CMD_PACKET_OK;	//exist good data
}

void Drv_Uart_Tx_arrange(void * data_ptr , uint16_t len)
{
	kdrv_uart_dev_id_t uart_dev = UART4_DEV;

	if(len == 1)
	{
		kdrv_uart_write(uart_dev, (uint8_t *)data_ptr, 1);
	}
	else if(len == 2)
	{
		kdrv_uart_write(uart_dev, ((uint8_t *)data_ptr)+1, 1);
		kdrv_uart_write(uart_dev, ((uint8_t *)data_ptr)+0, 1);
	}
	else if(len == 4)
	{
		kdrv_uart_write(uart_dev, ((uint8_t *)data_ptr)+3, 1);
		kdrv_uart_write(uart_dev, ((uint8_t *)data_ptr)+2, 1);
		kdrv_uart_write(uart_dev, ((uint8_t *)data_ptr)+1, 1);
		kdrv_uart_write(uart_dev, ((uint8_t *)data_ptr)+0, 1);
	}
	else
	{
		//error
	}
}

void Drv_Uart_Tx_send(struct kdp_cmd_adapter *p_cmd_adapter)
{
	uint16_t	i = 0;
	uint16_t	ntemp = 0;
	uint32_t	noffset =0;

	Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.head, sizeof(p_cmd_adapter->send_response.head));
	Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.host_number, sizeof(p_cmd_adapter->send_response.host_number));

  	if(p_cmd_adapter->send_response.status == CMD_STATUS_FAIL)
	{
		Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.status, sizeof(p_cmd_adapter->send_response.status));
		Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.tail, sizeof(p_cmd_adapter->send_response.tail));
		return;
	}

	if(p_cmd_adapter->send_response.cmd_stat == CMD_READ_NIR_ACT || p_cmd_adapter->send_response.cmd_stat == CMD_READ_RGB_ACT)
	{
		Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.status, sizeof(p_cmd_adapter->send_response.status));
		Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.sector_index, sizeof(p_cmd_adapter->send_response.sector_index));
		Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.data_size, sizeof(p_cmd_adapter->send_response.data_size));

		noffset = p_cmd_adapter->send_response.sector_index*SPI020_SECTOR_SIZE;
         
		//send read data
		//a lot of data
		for(i=0 ; i< *p_cmd_adapter->send_response.ptr_index; i++)
		{
            //DSG(" [OTA] RGB address 0x%x ", (uint32_t)(stota->send_response.ddr_ptr+ noffset + i ));
			ntemp = *((uint8_t *)p_cmd_adapter->send_response.ddr_ptr+ noffset + i );
			Drv_Uart_Tx_arrange((uint8_t *)&ntemp, 1);

			if((i%255 ==0) && (i>0))
			{
				osDelay(1);
			}
		}
		//send checksum
	    Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.check_sum, sizeof(p_cmd_adapter->send_response.check_sum));
	}
	//normal status
	else if(   p_cmd_adapter->send_response.cmd_stat == CMD_PROGRAM_ACT )
	{
		Drv_Uart_Tx_arrange( &p_cmd_adapter->send_response.status, sizeof(p_cmd_adapter->send_response.status) );
	}
	else if(  p_cmd_adapter->send_response.cmd_stat == CMD_READ_ACT )
	{
		Drv_Uart_Tx_arrange( &p_cmd_adapter->send_response.status, sizeof(p_cmd_adapter->send_response.status) );
		Drv_Uart_Tx_arrange( &p_cmd_adapter->send_response.sector_index, sizeof(p_cmd_adapter->send_response.sector_index) );
		Drv_Uart_Tx_arrange( &p_cmd_adapter->send_response.data_size, sizeof(p_cmd_adapter->send_response.data_size) );

		DSG("   [OTA] Tx send Tx_size.. pre 0x%x", *p_cmd_adapter->send_response.ptr_index);
		DSG("   [OTA] Data size  0x%x", p_cmd_adapter->send_response.data_size );


        //send read data
        //a lot of data
		*p_cmd_adapter->send_response.ptr_index = 0 ;

		for(  i=0 ; i< p_cmd_adapter->send_response.data_size; i++  )
		{
			ntemp = *( ( (uint8_t *)p_cmd_adapter->send_response.ptr ) + i );
			Drv_Uart_Tx_arrange( (uint8_t *)&ntemp, 1);
			*p_cmd_adapter->send_response.ptr_index = *p_cmd_adapter->send_response.ptr_index + 1 ;

			if( (i%255 ==0) && (i>0) )
			{
				kdrv_pwmtimer_delay_ms(1);
			}
		}

		DSG("   [OTA] Tx send Tx_size.. after 0x%x", *p_cmd_adapter->send_response.ptr_index);
		//send checksum
		Drv_Uart_Tx_arrange( &p_cmd_adapter->send_response.check_sum, sizeof(p_cmd_adapter->send_response.check_sum) );

	}
	else if( p_cmd_adapter->send_response.cmd_stat == CMD_ERASE_SECTOR_ACT  )
	{
		Drv_Uart_Tx_arrange( &p_cmd_adapter->send_response.status, sizeof(p_cmd_adapter->send_response.status) );
	}
	else if(p_cmd_adapter->send_response.cmd_stat == CMD_ACT)
	{
		Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.status, sizeof(p_cmd_adapter->send_response.status) );
	}
    Drv_Uart_Tx_arrange(&p_cmd_adapter->send_response.tail, sizeof(p_cmd_adapter->send_response.tail));
	return;
}


void Drv_OTA_packet_response(struct kdp_cmd_adapter *p_cmd_adapter, uint8_t status )
{
	uint8_t	*ptr = p_cmd_adapter->send_response.ptr;
	uint32_t	*ptr_indedx = p_cmd_adapter->send_response.ptr_index;
	uint8_t	ncmd = p_cmd_adapter->receive_cmd.cmd_stat;
	uint32_t	nchecksum = 0;

	p_cmd_adapter->send_response.head = ~(CMD_HEAD_RX);
	p_cmd_adapter->send_response.host_number = p_cmd_adapter->receive_cmd.host_number;
	p_cmd_adapter->send_response.tail = CMD_TAIL;
	p_cmd_adapter->send_response.action_number = p_cmd_adapter->receive_cmd.action_number;

	//--- error message
	if(status == CMD_PACKET_HEAD_CHECK_ERROR || status == CMD_PACKET_TAIL_CHECK_ERROR || status == CMD_DRV_FAIL)
	{
		p_cmd_adapter->send_response.status = CMD_STATUS_FAIL;
	}
	else
	{
		//--- program or read
		p_cmd_adapter->send_response.status = CMD_STATUS_OK;
		p_cmd_adapter->send_response.cmd_stat = ncmd;

		if( ncmd == CMD_PROGRAM_ACT || ncmd == CMD_ACT || ncmd == CMD_ERASE_SECTOR_ACT )
		{
			p_cmd_adapter->send_response.sector_index = 0;
			p_cmd_adapter->send_response.data_size = 0;
			p_cmd_adapter->send_response.check_sum = 0;
		}

		if( ncmd == CMD_READ_ACT )
		{

			p_cmd_adapter->send_response.sector_index = p_cmd_adapter->receive_cmd.sector_index;
			p_cmd_adapter->send_response.data_size = *p_cmd_adapter->send_response.ptr_index;


			DSG("[OTA] Read checksum 1, size: %d",p_cmd_adapter->send_response.data_size  );
			ptr = p_cmd_adapter->receive_cmd.ptr;
			nchecksum = Drv_utility_checksum( ptr , 0, p_cmd_adapter->send_response.data_size );

			DSG("[OTA] Read checksum 2");

			nchecksum+= ( (p_cmd_adapter->send_response.data_size&0xFF)  +  (p_cmd_adapter->send_response.data_size>>8)&0xFF );
			nchecksum+= ( (p_cmd_adapter->send_response.sector_index&0xFF) + (p_cmd_adapter->send_response.sector_index>>8)&0xFF );
			nchecksum+= ((p_cmd_adapter->send_response.status&0xFF) + (p_cmd_adapter->send_response.status>>8)&0xFF );
			p_cmd_adapter->send_response.check_sum = nchecksum;
		}

		if(ncmd == CMD_READ_NIR_ACT)
		{
			p_cmd_adapter->send_response.sector_index = p_cmd_adapter->receive_cmd.sector_index;
			p_cmd_adapter->send_response.data_size = *p_cmd_adapter->send_response.ptr_index;
			p_cmd_adapter->send_response.ddr_ptr = p_cmd_adapter->receive_cmd.ddr_ptr;
            if(p_cmd_adapter->send_response.ddr_ptr == 0)
            {
                p_cmd_adapter->send_response.ddr_ptr = kl520_api_get_snapshot_addr(CAMERA_NIR_IDX);
            }                
            
			nchecksum = Drv_utility_checksum((uint8_t *)p_cmd_adapter->send_response.ddr_ptr + p_cmd_adapter->send_response.sector_index*SPI020_SECTOR_SIZE			//ddr_ptr+dector_index*4096 = real_address
																, 0, *ptr_indedx );

			nchecksum += ((p_cmd_adapter->send_response.data_size&0xFF)  +  (p_cmd_adapter->send_response.data_size>>8)&0xFF );
			nchecksum += ((p_cmd_adapter->send_response.sector_index&0xFF) + (p_cmd_adapter->send_response.sector_index>>8)&0xFF );
			nchecksum += ((p_cmd_adapter->send_response.status&0xFF) + (p_cmd_adapter->send_response.status>>8)&0xFF );
			p_cmd_adapter->send_response.check_sum = nchecksum;
		}

		if( ncmd == CMD_READ_RGB_ACT )
		{            
			p_cmd_adapter->send_response.sector_index = p_cmd_adapter->receive_cmd.sector_index;
			p_cmd_adapter->send_response.data_size = *p_cmd_adapter->send_response.ptr_index;
			p_cmd_adapter->send_response.ddr_ptr = p_cmd_adapter->receive_cmd.ddr_ptr;
            if(p_cmd_adapter->send_response.ddr_ptr == 0)
            {
                p_cmd_adapter->send_response.ddr_ptr = kl520_api_get_snapshot_addr(CAMERA_RGB_IDX);
            }   
            
			nchecksum = Drv_utility_checksum( (uint8_t *)p_cmd_adapter->send_response.ddr_ptr + p_cmd_adapter->send_response.sector_index*SPI020_SECTOR_SIZE			//ddr_ptr+dector_index*4096 = real_address
																				, 0, *ptr_indedx );
			nchecksum+= ((p_cmd_adapter->send_response.data_size&0xFF)  +  (p_cmd_adapter->send_response.data_size>>8)&0xFF );
			nchecksum+= ((p_cmd_adapter->send_response.sector_index&0xFF) + (p_cmd_adapter->send_response.sector_index>>8)&0xFF );
			nchecksum+= ((p_cmd_adapter->send_response.status&0xFF) + (p_cmd_adapter->send_response.status>>8)&0xFF );
			p_cmd_adapter->send_response.check_sum = nchecksum;
		}
	}

	//-------------- packet data into Uart FIFO ----------------------
	Drv_Uart_Tx_send(p_cmd_adapter);

	return;
}

static uint8_t kdp_cmd_adapter_parser(struct kdp_cmd_adapter *p_cmd_adapter)
{
	uint32_t	addr;
	uint16_t	ntime_out = 0;
	uint16_t	ntime_out_target = 5;
	uint16_t	ntime_out_interval = 100;		//1000ms is 1s

	addr = p_cmd_adapter->receive_cmd.sector_index * SPI020_SECTOR_SIZE;//st_flash_info.sector_size_Bytes;

    switch(p_cmd_adapter->receive_cmd.cmd_stat)
	{
		case cmd_parser_init:
			//action number check
			if((p_cmd_adapter->receive_cmd.action_number == CMD_ACT_NUM_INIT) ||
				(p_cmd_adapter->receive_cmd.action_number == CMD_ACT_NUM_SW_VERSION))
			{
                DSG("   [OTA] flash init ");
				kdev_flash_initialize();
				if(kdev_flash_get_info() == 0)
				{
					return CMD_DRV_FAIL;
				}
                DSG("   [OTA] flash init done ");
			}
			if((p_cmd_adapter->receive_cmd.action_number == CMD_ACT_NUM_NIR) ||
                (p_cmd_adapter->receive_cmd.action_number == CMD_ACT_NUM_RGB))
			{
				//call active
                kdp_camera_enable_snapshot_fdfr();
			}
			break;

		case cmd_parser_flash_program:

			//need add a timeout mechanism ??
            DSG("   [OTA] flash program addr %d ", addr);
			if( kdev_flash_programdata( addr , p_cmd_adapter->receive_cmd.ptr+p_cmd_adapter->receive_cmd.data_offset
																   , p_cmd_adapter->receive_cmd.data_size ) != 0 )
			{
				return CMD_DRV_FAIL;
			}
            DSG("   [OTA] flash program done");
			break;

		case cmd_parser_flash_read:
			#if ( OTA_SPI_EN == 1 )
				*p_cmd_adapter->send_response.ptr_index = 0;		//clear buf index
				*p_cmd_adapter->receive_cmd.ptr_index = 0;			//clear buf index
				if( kdev_flash_readdata( addr , p_cmd_adapter->receive_cmd.ptr , p_cmd_adapter->receive_cmd.ptr_index, p_cmd_adapter->receive_cmd.data_size ) == 0 )
				{
					return CMD_DRV_FAIL;
				}
				p_cmd_adapter->send_response.data_size = p_cmd_adapter->receive_cmd.data_size;
				*p_cmd_adapter->send_response.ptr_index = p_cmd_adapter->receive_cmd.data_size;

			#else
				DSG("   [OTA] flash read addr %d ", addr);
				*p_cmd_adapter->send_response.ptr_index = 0;		//clear buf index
                if( kdev_flash_readdata( addr , p_cmd_adapter->send_response.ptr , p_cmd_adapter->receive_cmd.data_size ) != 0 )
				{
					return CMD_DRV_FAIL;
				}
				p_cmd_adapter->send_response.data_size = p_cmd_adapter->receive_cmd.data_size;
				*p_cmd_adapter->send_response.ptr_index = p_cmd_adapter->receive_cmd.data_size;
				DSG("   [OTA] flash read done ");
			#endif
		break;

		case cmd_parser_flash_erase:
            DSG("   [OTA] flash erase ");
            DSG("_flash_erase_multi_sectors start_addr = %d! end_addr = %d!", p_cmd_adapter->receive_cmd.sector_index, p_cmd_adapter->receive_cmd.data_offset);
			if ( kdev_flash_erase_multi_sector( p_cmd_adapter->receive_cmd.sector_index, p_cmd_adapter->receive_cmd.data_offset ) != 0 )
			{
				return CMD_DRV_FAIL;
			}
            DSG("   [OTA] flash erase done ");
			break;

		case cmd_parser_NIR:
			if((p_cmd_adapter->receive_cmd.ddr_ptr = kl520_api_get_snapshot_addr(CAMERA_NIR_IDX)) == 0 )
			{
				osDelay(ntime_out_interval);
				//call wait
				ntime_out++;
                osDelay(ntime_out_interval);
				if(ntime_out > ntime_out_target)
				{
					return CMD_DRV_FAIL;
				}
			}
			*p_cmd_adapter->send_response.ptr_index = p_cmd_adapter->receive_cmd.data_size;
			break;

		case cmd_parser_RGB:
			if((p_cmd_adapter->receive_cmd.ddr_ptr = kl520_api_get_snapshot_addr(CAMERA_RGB_IDX))== 0)
			{
				osDelay(ntime_out_interval);
				//call wait
				ntime_out++;
                osDelay(ntime_out_interval);
				if(ntime_out > ntime_out_target)
				{
					return CMD_DRV_FAIL;
				}
			}
			*p_cmd_adapter->send_response.ptr_index = p_cmd_adapter->receive_cmd.data_size;
			break;

		default:
			break;
	}
	return CMD_DRV_OK;
}

/**
* @brief  This is a command adapter which will always polls UART4 rx buffer
*
* @param  N/A
* 
* @return N/A
*/
static void kdp_cmd_adapter_thread(void *argument)
{
    uint32_t    countinuous_empty_count = 0;
    uint32_t    countinuous_empty_target = 10;

    static uint8_t  nstat;
    static uint16_t ptr_rx_count = 0;
    static uint16_t nretry_acc = 0;
    uint16_t nretry_acc_target = 255;

    while(1)
    {
        //dbg_msg("[OTA] Start !!");
        if( *m_cmd_adapter.receive_cmd.ptr_index == 0  )
        {
            countinuous_empty_count++;
            if(countinuous_empty_count > countinuous_empty_target) {
                osDelay( 1000 );
            }
            else {
                osDelay( 2 );
            }
        }
        else
        {
            countinuous_empty_count = 0;
        }

        switch(m_cmd_adapter.flow)
        {
            case cmd_init:
                Drv_OTA_init(&m_cmd_adapter);
                m_cmd_adapter.flow = cmd_idle;
                break;

            case cmd_packet_analyze:
                *m_cmd_adapter.receive_cmd.ptr_index = kdrv_uart_get_rx_count(UART4_DEV);
                if(*m_cmd_adapter.receive_cmd.ptr_index == 0)
                {
                    break;//    m_cmd_adapter.flow = cmd_idle; //sleep or others!!
                }
 
                if((nstat = Drv_OTA_packet_analyze(&m_cmd_adapter)) == CMD_PACKET_OK)
                {
                     nstat = kdp_cmd_adapter_parser(&m_cmd_adapter) ;
                     m_cmd_adapter.receive_cmd.busy_flag = 0;
                     m_cmd_adapter.flow = cmd_response_doing;
                }
                else
                {
                     m_cmd_adapter.receive_cmd.busy_flag = 0;
                     if(nstat == CMD_PACKET_HEAD_CHECK_ERROR)
                     {
                         //goto report error
                         m_cmd_adapter.flow = cmd_response_doing;       // eOTA_error_report;
                     }

                     if(nstat == CMD_PACKET_TAIL_CHECK_ERROR)
                     {
                         if(ptr_rx_count > 0  &&  ptr_rx_count == *m_cmd_adapter.receive_cmd.ptr_index)
                         {
                             nretry_acc ++;
                             kdrv_pwmtimer_delay_ms(1);
                             if(nretry_acc >= nretry_acc_target)
                             {
                                 nretry_acc = 0;
                                 m_cmd_adapter.flow = cmd_response_doing;
                             }
                             else
                             {
                                 //m_cmd_adapter.flow = cmd_packet_analyze;     //any good case
                             }
                         }
                         ptr_rx_count = *m_cmd_adapter.receive_cmd.ptr_index;
                     }
                }
                break;

             case cmd_response_doing:
                 Drv_OTA_packet_response(&m_cmd_adapter, nstat) ;            
                 m_cmd_adapter.flow = cmd_idle;
                 break;

             case cmd_error_report:
                 m_cmd_adapter.flow = cmd_idle;
                 break;
 
             case cmd_idle:
                 nretry_acc = 0;
                 kdrv_uart_read(UART4_DEV, g_uart_Rx_buffer, UART_RX_BUFFER_SIZE);         //read case
                 *m_cmd_adapter.receive_cmd.ptr_index = 0 ;
                 *m_cmd_adapter.send_response.ptr_index = 0;
                 m_cmd_adapter.flow = cmd_packet_analyze;
                 break;
            
             default:
                 break;
         }
     }
}

extern void kdp_uart_app_com4(void);
void kdp_cmd_adapter_init(void)
{
    if (cmd_adapter_tid != NULL)
        return;

    kdp_uart_app_com4();
    kdrv_uart_read(UART4_DEV, g_uart_Rx_buffer, UART_RX_BUFFER_SIZE);

    m_cmd_adapter.flow = cmd_init;
    if((cmd_adapter_tid = osThreadNew(kdp_cmd_adapter_thread, NULL, NULL)) != NULL) {
        osThreadSetPriority(kdp_cmd_adapter_thread , osPriorityNormal);
    }
}


