/*
 * @name : v2k_dev_ops.c
 * @brief : Video capture interface for Mozart
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
#include <stdlib.h>
#include <string.h>
#include "framework/framework_errno.h"
#include "media/videodev.h"
#include "kmdw_console.h"

#define V2K_MAX_DEVICE 2

struct v2k_dev_handle g_dev_handles[V2K_MAX_DEVICE] = { 0 } ;

static int v2k_open(struct v2k_dev_handle *handle)
{
    int ret = -KDP_FRAMEWORK_ERRNO_CTX;    
    struct video_device_context *vdc = video_devdata(handle);

    DSG("[%s] vdc=%p", __func__, vdc);

    if ((vdc) &&(vdc->dev_ops->open)) {
        ret = vdc->dev_ops->open(handle);
    }
    
    return ret;
}

static int v2k_release(struct v2k_dev_handle *handle)
{
    int ret = 0;    
    struct video_device_context *vdc = video_devdata(handle);

    //DSG("[%s]", __func__);
    if (vdc->dev_ops->release)
        ret = vdc->dev_ops->release(handle);

    return ret;
}
    
static long v2k_ioctl(struct v2k_dev_handle *handle, 
        unsigned int cmd, void *arg)
{
    int ret = -KDP_FRAMEWORK_ERRNO_CTX;
    struct video_device_context *vdc = video_devdata(handle);

    if (vdc->dev_ops->ioctl) {
        ret = vdc->dev_ops->ioctl(handle, cmd, arg);
    } 

    return ret;
}

const struct v2k_dev_operations v2k_dev_ops = {
    .open = v2k_open,
    .release = v2k_release,
    .ioctl = v2k_ioctl,    
};

const struct v2k_dev_operations* v2k_get_dev_ops(void) {
    return &v2k_dev_ops;
} 

struct v2k_dev_handle* v2k_get_dev_handle(int fd) {
    //DSG("[%s] fd=%d", __func__, fd);
    return &g_dev_handles[fd - 1];
}

void v2k_set_dev_handle(struct v2k_dev_handle *handle) {
    //DSG("[%s] handle->i_rdev=%d", __func__, handle->i_rdev);
    g_dev_handles[handle->i_rdev].i_rdev = handle->i_rdev;
    g_dev_handles[handle->i_rdev].dev_ops = &v2k_dev_ops;
}
