#ifndef __DRIVER_H__
#define __DRIVER_H__


#include <stddef.h>
#include "base.h"
//#include "framework/init.h"
#include "framework/kdp_list.h"
#include "framework/v2k_image.h"


struct pin_context {
    void *platform_data;
    void *driver_data;
    void *driver_this;
};
struct pin_context2 {
    void *platform_data;
};

struct driver_context {
    char name[20];

    int (*add) (struct driver_context *, struct pin_context *);
    int (*del) (struct driver_context *, struct pin_context *);

    struct kdp_list_node knode_to_buslist;
};

struct driver_context2 {
    int (*probe)(struct driver_context2 *, struct pin_context *);
    int (*reset)(struct driver_context2 *, struct pin_context *);
};

struct kdp520_csi2rx_context_lite {
    u32 ai_mem_addr;
    u32 ai_mem_addr_tmp;
    u32 vw_mem_addr;
    u32 ov_mem_addr;
    u32 dpi2ahb_base;
};
struct core_device {
    const char *name;
    struct ioport_setting *ioport;
    unsigned int num_ioports;
    int uuid; // universal unique id
    struct pin_context pin_ctx;
};
#define to_core_device(x) container_of((x), struct core_device, pin_ctx)

/* for core driver extension */
struct power_mgr_api { 
    int (*power)(struct core_device *, bool );/* power on or off the device */
    int (*suspend)(struct core_device *);   /* power management - suspend */
    int (*resume)(struct core_device *);    /* power management - resume */    
};

struct kdp520_i2c_params {
    bool public_mode;
    unsigned long bus_speed;
};
struct core_i2c_driver {
    int (*probe)(struct core_device *);     /* used to set up device-specific structures or resource */
    int (*remove)(struct core_device *);    /* used to release device-specfic structures or resource */
    struct core_device *core_dev;
    struct driver_context driver;
    struct power_mgr_api power_mgr;
    void *cfg_func;

    int (*set_params)(struct core_device *, struct kdp520_i2c_params *);
    int (*init)(struct core_device *);
    int (*reset)(struct core_device *);
    int (*write)(struct core_device *, u8 , u16 , u16 , u8 );
    int (*read)(struct core_device *, u8 , u16 , u16 , u8 *);
    int (*readbytes)(struct core_device *, u8 , u16 , u16 , u8 *, u8);
    
    bool inited;
    struct kdp520_i2c_params params;
};

////////////////////////////
/* specific platform data */
////////////////////////////
/* i2c platform data */
struct i2c_platform_data {
    unsigned long bus_speed;
};


/* display platform data */
//struct fb_platform_data {
//    unsigned long input_fmt; // refer to enum image_input_format definition
//    unsigned short input_xres;
//    unsigned short input_yres; 
//};


extern struct driver_context* find_driver(char *);
extern struct ioport_setting *driver_get_ioport_setting(struct core_device *, unsigned int);

extern int drivers_init(void);
extern int driver_register(struct driver_context *, struct pin_context *);
extern void driver_unregister(struct driver_context *);
//extern int driver_core_register(struct core_driver *);
//extern void driver_core_unregister(struct core_driver *);
extern int driver_core_register(void *);
extern void driver_core_unregister(void *);

void* framework_drv_get_drvdata(const struct pin_context *);
void  framework_drv_set_drvdata(struct pin_context *, void *data);
void* framework_drv_get_drv(const struct pin_context *pin_ctx);
void  framework_drv_set_drv(struct pin_context *pin_ctx, void *this);

#define DRIVER_SETUP(__driver, __setup_level, __register, __unregister, ...) \
static int INITTEXT __driver##_entr(void) { \
    return __register(&(__driver) , ##__VA_ARGS__); \
} \
ARRANGE_ENTR_RO_SECTION(__driver##_entr, __setup_level); \
static void FINITEXT __driver##_exit(void) { \
    __unregister(&(__driver) , ##__VA_ARGS__); \
} \
ARRANGE_EXIT_RO_SECTION(__driver##_exit);

#define DRIVER_SETUP_2(__driver, __setup_level, __register, __unregister, ...) \
static int INITTEXT __driver##_entr(void) { \
    return __register((struct core_driver*)(&(__driver)), ##__VA_ARGS__); \
} \
ARRANGE_ENTR_RO_SECTION(__driver##_entr, __setup_level); \
static void FINITEXT __driver##_exit(void) { \
    __unregister((struct core_driver*)(&(__driver)), ##__VA_ARGS__); \
} \

#define KDP_CORE_DRIVER_SETUP(__core_driver, __setup_level) \
    DRIVER_SETUP(__core_driver, __setup_level, driver_core_register, driver_core_unregister)
#define KDP_CORE_DRIVER_SETUP_2(__core_driver, __setup_level) \
    DRIVER_SETUP_2(__core_driver, __setup_level, driver_core_register, driver_core_unregister)

int framework_drv_init(void);

#endif /* __DEVICE_H__ */
