/**
 * Copyright (C) 2020 Kneron Inc. All right reserved
 *
 * @file kapp_ops.h
 * @brief Kneron Application Shared Data
 */

#ifndef __KAPP_OPS_H__
#define __KAPP_OPS_H__

#include "cmsis_os2.h"
#include "ipc.h"
#include "kapp_id.h"

enum kapp_ops_status {
    KAPP_OP_OK = 0,
    KAPP_ABORT,
    KAPP_ERR,
};

struct kapp_img_run_s {
    /* Input Image configuration struct */
    struct kdp_img_cfg      img_cfg;

    /* Image processing parameters */
    struct kdp_crop_box_s   *crop_box;
    struct kdp_pad_value_s  *pad_values;
    void                    *ext_param;

    /* App specific result struct output buffer */
    void                    *out;

    /* Output buffer size defined by host side in companion mode */
    uint32_t                out_size;

    /* Event to receive final result later in parallel mode */
    osEventFlagsId_t    evt_id;
    uint32_t            evt_flag;
};

struct kapp_ops {
    int (*start)(uint32_t app_id);
    int (*run_image)(uint32_t app_id, struct kapp_img_run_s *img_run_p);
    int (*get_result_len)(uint32_t app_id, void *result_buf);
};

#endif // __KAPP_OPS_H__
