/******************************************************************************
*  Copyright 2020, Kneron, Inc.
*  ---------------------------------------------------------------------------
*  Statement:
*  ----------
*  This software is protected by Copyright and the information contained
*  herein is confidential. The software may not be copied and the information
*  contained herein may not be used or disclosed except with the written
*  permission of Kneron, Inc. (C) 2020
******************************************************************************/

/******************************************************************************
*  Filename:
*  ---------
*  kdrv_mipicsirx.h
*
*  Project:
*  --------
*  KL520
*
*  Description:
*  ------------
*  Configuration for KL520
*
*  Author:
*  -------
*  Albert Chen
*
*===========================================================================
*
******************************************************************************/
/**@addtogroup  KDRV_MIPICSIRX  KDRV_MIPICSIRX
 * @{
 * @brief       Kneron mipicsirx driver
 * @copyright   Copyright (C) 2020 Kneron, Inc. All rights reserved.
 */

#ifndef _KDRV_MIPICSIRX_H_
#define _KDRV_MIPICSIRX_H_
/******************************************************************************
Head Block of The File
******************************************************************************/
#include "kdrv_status.h"
enum {
    CSI2RX_CAM_0,
    CSI2RX_CAM_1,
    CSI2RX_CAM_NUM,    // = IMGSRC_NUM
};
/**
* @brief       kdrv_csi2rx_initialize
*              Initiclize mipicsirx related variable.
* @param[in]   cam_idx      cam idx
* @return      kdrv_status_t
*/
kdrv_status_t kdrv_csi2rx_initialize(uint32_t cam_idx);

/**
* @brief       kdrv_csi2rx_enable
*              Set mipicsirx related register for IP enable.
* @param[in]   input_type   sensor input type
* @param[in]   cam_idx      cam idx
* @param[in]   sensor_idx   sensor idx
* @param[in]   fmt          camera related format setting
* @return      kdrv_status_t
*/
kdrv_status_t kdrv_csi2rx_enable(uint32_t input_type, uint32_t cam_idx, uint32_t sensor_idx, struct cam_format* fmt);

/**
* @brief       kdrv_csi2rx_start
*              Set mipicsirx related register for IP start.
* @param[in]   input_type   sensor input type
* @param[in]   cam_idx      cam_idx
* @return      kdrv_status_t
*/
kdrv_status_t kdrv_csi2rx_start(uint32_t input_type, uint32_t cam_idx);

/**
* @brief       kdrv_csi2rx_set_power
*              Set mipicsirx power related register.
* @param[in]   cam_idx      cam_idx
* @param[in]   on           csirx power status, 1: ON, 0:Off
* @return      kdrv_status_t
*/
kdrv_status_t kdrv_csi2rx_set_power(uint32_t cam_idx, uint32_t on);

/**
* @brief       kdrv_csi2rx_reset
*              Reset mipicsirx.
* @param[in]   cam_idx      cam_idx
* @param[in]   on           csirx power status, 1: ON, 0:Off
* @return      kdrv_status_t
*/
kdrv_status_t kdrv_csi2rx_reset(uint32_t cam_idx, uint32_t sensor_idx);


#endif // _KDRV_MIPICSIRX_H_
