/********************************************************************
 * Copyright (c) 2020 Kneron, Inc. All Rights Reserved.
 *
 * The information contained herein is property of Kneron, Inc.
 * Terms and conditions of usage are described in detail in Kneron
 * STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information.
 * NO WARRANTY of ANY KIND is provided. This heading must NOT be removed
 * from the file.
 ********************************************************************/

/**@addtogroup  KDRV_NPU KDRV_NPU
 * @{
 * @brief       Kneron NPU driver
 * @copyright   Copyright (C) 2020 Kneron, Inc. All rights reserved.
 */

#ifndef __KDRV_NPU_H_
#define __KDRV_NPU_H_

#include "cmsis_os2.h"
#include "ipc.h"

typedef void (*ipc_handler_t)(int ipc_idx, int state);

/**
 * @brief Initialize NPU functionality
 * @param ipc_handler IPC callback
 */
void kdrv_ncpu_initialize(ipc_handler_t ipc_handler);

/**
 * @brief Set model information
 * @param model_info_addr model information address
 * @param info_idx information index
 * @param slot_idx slot index
 */
void kdrv_ncpu_set_model(struct kdp_model_s *model_info_addr, uint32_t info_idx, int32_t slot_idx);

/**
 * @brief Get available COM
 * @return COM index
 */
int kdrv_ncpu_get_avail_com(void);

/**
 * @brief Set active model index
 * @param ipc_idx IPC index
 * @param index model slot index
 */
void kdrv_ncpu_set_model_active(int ipc_idx, uint32_t index);

/**
 * @brief Set active image index
 * @param index image index
 * @return available COM
 */
int kdrv_ncpu_set_image_active(uint32_t index);

/**
 * @brief Set SCPU debug level
 * @param lvl level
 */
void kdrv_ncpu_set_scpu_debug_lvl(uint32_t lvl);

/**
 * @brief Set NCPU debug level
 * @param lvl level
 */
void kdrv_ncpu_set_ncpu_debug_lvl(uint32_t lvl);

/**
 * @brief Trigger NCPU interrupt
 * @param ipc_idx IPC channel to trigger
 */
void kdrv_ncpu_trigger_int(int ipc_idx);

/**
 * @brief Get ncpu_to_scpu_s
 * @return IPC struct
 */
struct ncpu_to_scpu_s* kdrv_ncpu_get_input(void);

/**
 * @brief Get scpu_to_ncpu_s
 * @return IPC struct
 */
struct scpu_to_ncpu_s* kdrv_ncpu_get_output(void);

#endif
