/* Copyright (c) 2020 Kneron, Inc. All Rights Reserved.
 *
 * The information contained herein is property of Kneron, Inc.
 * Terms and conditions of usage are described in detail in Kneron
 * STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information.
 * NO WARRANTY of ANY KIND is provided. This heading must NOT be removed
 * from the file.
 */

/******************************************************************************
*  Filename:
*  ---------
*  kdrv_pinmux.c
*
*  Project:
*  --------
*  KL520
*
*  Description:
*  ------------
*  This is pinmux config driver
*
*  Author:
*  -------
*  XXX
**
******************************************************************************/

/******************************************************************************
Head Block of The File
******************************************************************************/

/* Comment block of the file */

// Sec 1: Include File

#ifdef USE_KDRV
#include <stdarg.h>
#include "kdrv_scu_ext.h"
#include "base.h"
#include "kdrv_pinmux.h"

// Sec 2: Constant Definitions, Imported Symbols, miscellaneous
#define PIN_NUM                                 38
#define KDRV_PIN_SPI_WP_N_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_SPI_HOLD_N_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_JTAG_TRST_N_REG                PIN_MODE_0 | (PIN_PULL_DOWN << 3) | (PIN_DRIVING_12MA << 6) //0x00000090
#define KDRV_PIN_JTAG_TDI_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_JTAG_SWDITMS_REG               PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_JTAG_SWCLKTCK_REG              PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_JTAG_TDO_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_LC_PCLK_REG                    PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_4MA  << 6) //0x00000000
#define KDRV_PIN_LC_VS_REG                      PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_HS_REG                      PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DE_REG                      PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_0_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_4MA  << 6) //0x00000000
#define KDRV_PIN_LC_DATA_1_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_4MA  << 6) //0x00000000
#define KDRV_PIN_LC_DATA_2_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_4MA  << 6) //0x00000000
#define KDRV_PIN_LC_DATA_3_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_4_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_5_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_6_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_7_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_8_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_9_REG                  PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_10_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_11_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_12_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_13_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_14_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_LC_DATA_15_REG                 PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_8MA  << 6) //0x00000040
#define KDRV_PIN_SD_CLK_REG                     PIN_MODE_1 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000081
#define KDRV_PIN_SD_CMD_REG                     PIN_MODE_1 | (PIN_PULL_UP   << 3) | (PIN_DRIVING_12MA << 6) //0x00000089
#define KDRV_PIN_SD_DAT_0_REG                   PIN_MODE_0 | (PIN_PULL_UP   << 3) | (PIN_DRIVING_4MA  << 6) //0x00000008
#define KDRV_PIN_SD_DAT_1_REG                   PIN_MODE_0 | (PIN_PULL_UP   << 3) | (PIN_DRIVING_4MA  << 6) //0x00000008
#define KDRV_PIN_SD_DAT_2_REG                   PIN_MODE_0 | (PIN_PULL_UP   << 3) | (PIN_DRIVING_4MA  << 6) //0x00000008
#define KDRV_PIN_SD_DAT_3_REG                   PIN_MODE_0 | (PIN_PULL_UP   << 3) | (PIN_DRIVING_4MA  << 6) //0x00000008
#define KDRV_PIN_UART0_RX_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_UART0_TX_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080
#define KDRV_PIN_I2C0_SCL_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_4MA  << 6) //0x00000000
#define KDRV_PIN_I2C0_SDA_REG                   PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_4MA  << 6) //0x00000000
#define KDRV_PIN_PWM0_REG                       PIN_MODE_0 | (PIN_PULL_NONE << 3) | (PIN_DRIVING_12MA << 6) //0x00000080

/******************************************************************************
Declaration of External Variables & Functions
******************************************************************************/
// Sec 3: declaration of external variable

// Sec 4: declaration of external function prototype


/******************************************************************************
Declaration of data structure
******************************************************************************/
// Sec 5: structure, uniou, enum, linked list

/******************************************************************************
Declaration of Global Variables & Functions
******************************************************************************/
// Sec 6: declaration of global variable

// Sec 7: declaration of global function prototype

/******************************************************************************
Declaration of static Global Variables & Functions
******************************************************************************/
// Sec 8: declaration of static global variable
static uint32_t Pinmux_array[PIN_NUM] ={ KDRV_PIN_SPI_WP_N_REG, KDRV_PIN_SPI_HOLD_N_REG, KDRV_PIN_JTAG_TRST_N_REG, KDRV_PIN_JTAG_TDI_REG, KDRV_PIN_JTAG_SWDITMS_REG, KDRV_PIN_JTAG_SWCLKTCK_REG, KDRV_PIN_JTAG_TDO_REG, KDRV_PIN_LC_PCLK_REG, KDRV_PIN_LC_VS_REG, KDRV_PIN_LC_HS_REG, KDRV_PIN_LC_DE_REG, KDRV_PIN_LC_DATA_0_REG, KDRV_PIN_LC_DATA_1_REG, KDRV_PIN_LC_DATA_2_REG, KDRV_PIN_LC_DATA_3_REG, KDRV_PIN_LC_DATA_4_REG, KDRV_PIN_LC_DATA_5_REG, KDRV_PIN_LC_DATA_6_REG, KDRV_PIN_LC_DATA_7_REG, KDRV_PIN_LC_DATA_8_REG, KDRV_PIN_LC_DATA_9_REG, KDRV_PIN_LC_DATA_10_REG, KDRV_PIN_LC_DATA_11_REG, KDRV_PIN_LC_DATA_12_REG, KDRV_PIN_LC_DATA_13_REG, KDRV_PIN_LC_DATA_14_REG, KDRV_PIN_LC_DATA_15_REG, KDRV_PIN_SD_CLK_REG, KDRV_PIN_SD_CMD_REG, KDRV_PIN_SD_DAT_0_REG, KDRV_PIN_SD_DAT_1_REG, KDRV_PIN_SD_DAT_2_REG, KDRV_PIN_SD_DAT_3_REG, KDRV_PIN_UART0_RX_REG, KDRV_PIN_UART0_TX_REG, KDRV_PIN_I2C0_SCL_REG, KDRV_PIN_I2C0_SDA_REG, KDRV_PIN_PWM0_REG};

// Sec 9: declaration of static function prototype

/******************************************************************************
// Sec 10: C Functions
******************************************************************************/
void kdrv_pinmux_init(void)
{
    for(uint32_t i = 0; i < PIN_NUM; i++)
    {
        uint32_t reg_addr = SCU_EXTREG_SPI_WP_N_IOCTRL + (i * 4);
        SET_MASKED_BITS(reg_addr, Pinmux_array[i], 0, 7)
    }
}
void kdrv_pinmux_config(kdrv_pin_name pin, kdrv_pinmux_mode mode, kdrv_pin_pull pull_type, kdrv_pin_driving driving)
{
    uint32_t reg_addr = SCU_EXTREG_SPI_WP_N_IOCTRL + (pin * 4);
    uint32_t ioctrl_bits = 0;

    /*Pinmux mode configure*/
    ioctrl_bits |= mode;

    /*Pull up, Pull down configure*/
    if(pull_type == PIN_PULL_UP)
        ioctrl_bits |= BIT3;
    else if(pull_type == PIN_PULL_DOWN)
        ioctrl_bits |= BIT4;

    /*Output Driving configure*/
    if(driving == PIN_DRIVING_8MA)
        ioctrl_bits |= BIT6;
    else if(driving == PIN_DRIVING_12MA)
        ioctrl_bits |= BIT7;
    else if(driving == PIN_DRIVING_16MA)
        ioctrl_bits |= (BIT6 | BIT7);

    SET_MASKED_BITS(reg_addr, ioctrl_bits, 0, 7)
}
#endif
