#ifndef __OTA_UPDATE_H__
#define __OTA_UPDATE_H__

#include "base.h"
#include "cmsis_os2.h"

#define SCPU_PARTITION0_START_IN_FLASH    0x2000
#define NCPU_PARTITION0_START_IN_FLASH    0x18000
#define PARTITION_0_CFG_START_IN_FLASH    0X28000

#define SCPU_PARTITION1_START_IN_FLASH    0x41000
#define NCPU_PARTITION1_START_IN_FLASH    0x57000
#define PARTITION_1_CFG_START_IN_FLASH    0X67000

#define SCPU_START_ADDRESS          0x10102000
#define NCPU_START_ADDRESS          0x28000000

#define SCPU_IMAGE_SIZE             0x16000
#define NCPU_IMAGE_SIZE             0x10000
#define PARTITION_CFG_SIZE          32

#define SUCCESS 0
#define MSG_AUTH_FAIL 251
#define MSG_FLASH_FAIL 252
#define MSG_DATA_ERROR 253

/* function used to read OTA content */
typedef uint32_t (*FnReadData)(uint32_t addr, uint32_t img_size);

/* @brief Init OTA function
 * @param tmp_buf temp buffer: size should be >= VERIFY_BLK_SZ
 * @param fn_read_data function to read data
 * @return 0 on success
 */
int kmdw_ota_init(uint8_t* tmp_buf, FnReadData fn_read_data);

/**
 * @brief Update SCPU firmware
 * @return 0 on success
 */
int kmdw_ota_update_scpu(void);

/**
 * @brief Update NCPU firmware
 * @return 0 on success
 */
int kmdw_ota_update_ncpu(void);

/**
 * @brief Update model
 * @param size model size
 * @return 0 on success
 */
int kmdw_ota_update_model(uint32_t size);

/**
 * @brief Update spl
 * @param size spl size
 * @return 0 on success
 */
int kmdw_ota_update_spl(uint32_t size);

/**
 * @brief Switch active partition
 * @param partition
 * @return 0 on success
 */
int kmdw_ota_switch_active_partition(uint32_t partition);

/**
 * @brief get active SCPU partition ID
 * @return
 *   0 - partition 0
 *   1 - partition 1
 *  -1 - error condition (2 active partitions)
 */
int kmdw_ota_get_active_scpu_partition(void);

/**
 * @brief get active NCPU partition ID
 * @return
 *     0 - partition 0
 *     1 - partition 1
 *    -1 - error condition (2 active partitions)
 */
int kmdw_ota_get_active_ncpu_partition(void);

#endif
