/**
 * Copyright (C) Kneron Inc. All right reserved
 *
 * @file kapp_lw3d.h
 * @brief Kneron Application LW3D APIs
 */

#ifndef __KAPP_LW3D_H_
#define __KAPP_LW3D_H_

#include <stdint.h>
#include "kapp.h"
#include "kapp_fid.h"

#define NIR_IMG_SOURCE_CROP_H 480
#define NIR_IMG_SOURCE_CROP_W 480

#define FD_LARGE_W_THRESH     256

enum kapp_lw3d_status_code_e {
    KAPP_LW3D_WRONG_USAGE = 0x300,   //KAPP_LW3D_CODES
    KAPP_LW3D_NO_ALIVE
};

/**
 * @brief do face recognition for inference or register with liveness check
 * @param [inout] *p_inout_p
 *          { op;
 *            face_id; //as input
 *            user_id; //as input (or ouput) }
 *        op = KAPP_FID_OP_INFERENCE   : fd/fr + 1:n compare
 *             KAPP_FID_OP_REGISTER    : fd/fr + save to temp
 *             KAPP_FID_OP_EXTRACT     : fd/fr only
 *        mode = 1-5 : capture + fd/fr + register in DDR DB
 *        outputPtr: N/A
 * @param [out] p_fd_out_p fd result
 * @param [out] p_lm_out_p lm result
 * @param [out] p_fr_out_p fr result
 * @param [out] p_nir_fd_out_p fd result
 * @param [out] p_nir_lm_out_p lm result
 * @param [out] p_nir_fr_out_p fr result
 * @param [out] p_livenss_p liveness result
 * @return OK or Err code
 */
int32_t kapp_lw3d(kapp_fid_inout_t          *p_inout_p,
                  struct facedet_result_s   *p_fd_out_p,
                  struct landmark_result_s  *p_lm_out_p,
                  struct fr_result_s        *p_fr_out_p,
                  struct facedet_result_s   *p_nir_fd_out_p,
                  struct landmark_result_s  *p_nir_lm_out_p,
                  struct fr_result_s        *p_nir_fr_out_p,
                  bool                      *liveness);

/**
 * @brief do nir liveness
 * @param [in] p_lm_out_p detected face landmarks
 * @param [in] img_cfg img config
 * @return OK or Err code
 */
int32_t kapp_nir_lv( struct landmark_result_s   *p_lm_out_p,
                    struct kdp_img_cfg *img_cfg);


#endif
