/*
 * Kneron Host Communication driver
 *
 * Copyright (C) 2018 Kneron, Inc. All rights reserved.
 *
 */
#include <stdio.h>
#include <string.h>
#include "base.h"
#include "def_share.h"
#include "com.h"
#include "com_err.h"
#include "msg.h"
#include "kmdw_console.h"
#include "kcomm.h"
#include "kdp_ddr_table.h"
#include "kapp_age_gender.h"
#include "kapp_dme.h"
#include "kmdw_memory.h"
#include "kdrv_ncpu.h"

#define DEF_LOG_CATEG  "kcomm"
// flags for dme inference thread
#define FLAG_INFERENCE_START1           BIT(0)
#define FLAG_INFERENCE_START2           BIT(1)
#define FLAG_INFERENCE_DONE1            BIT(2)
#define FLAG_INFERENCE_DONE2            BIT(3)

#define KDP_COL_MIN             16 /* Bytes, i.e. 128 bits */

extern uint32_t image_pool;
extern dme_res *dme_od_out;
extern fd_age_gender_res  *dme_agg_out;
extern uint32_t dme_raw_out;
extern uint32_t dme_out_len;
extern uint32_t dme_raw_len;
uint32_t ack_idx;

struct kdp_dme_conf_tbl_s    *p_dme_config_tbl = 0;
uint32_t dme_raw_data = 0;  // ddr for relocation of raw data
volatile u16 session_id = 0;  // for asynchronous mode tracking
volatile u16 active_image = 0;
volatile u16 session_status[2];
volatile u16 completed_session[2];
volatile u16 active_session[2];
volatile u16 active_model_index[2];
volatile u16 async_enable[2];  // used to controll raw output data relocation 
volatile u16 DME_loop[2];  // used for debug

osThreadId_t tid_infer;
extern osThreadId_t tid_host_comm;

/* ###########################
 * ##    local functions    ##
 * ########################### */

static uint32_t round_up(uint32_t num){
    return ((num + (KDP_COL_MIN - 1)) & ~(KDP_COL_MIN - 1));
}

static int do_inference(int session_idx)
{
    int32_t ret, raw_len;
    struct kdp_dme_cfg *model_config = p_dme_config_tbl->dme_config;


    model_config += active_model_index[session_idx];  // point to the correct model
    uint32_t output_num = model_config->output_num;
    uint32_t output_offset = 0;
    DME_loop[session_idx] = 1;  // re-init the get status loop counter
    if (session_status[session_idx] != 2)
        err_msg("[INF] --- status update error [%d]\n", session_status[session_idx]);
    session_status[session_idx] = 1;  // inference state machine: 0 -> error, 1 -> started, 16 -> done

    if (model_config->img_conf.image_format & IMAGE_FORMAT_RAW_OUTPUT) {
        // TODO: get output length from current model
        uint32_t output_len = *(uint32_t*)(KDP_DDR_MODEL_INFO_TEMP + 6*sizeof(uint32_t));
        raw_len = (1 + 5*output_num)*sizeof(uint32_t) + output_len;
        if (dme_raw_out == 0) {  // allocate raw output result buffer
            dme_raw_out = kmdw_ddr_reserve(raw_len*2);
            dme_raw_len = raw_len;  // must need to work, can't re-allocate this for now
        }

        if (raw_len > dme_raw_len) { // check if we have DME structures set up?
            err_msg("The later dme raw output len [%u] larger than prev. one [%u] is prohibited!\n", raw_len, dme_raw_len);
            return -1;
        }

        if (session_idx) output_offset = dme_raw_len;
        ret = kapp_dme((dme_res*)(dme_raw_out+output_offset), model_config);
    } else if (model_config->img_conf.image_format & IMAGE_FORMAT_MODEL_AGE_GENDER) {
        if (dme_agg_out == 0) {
            dme_agg_out = (fd_age_gender_res*)kmdw_ddr_reserve(sizeof(fd_age_gender_res)*2);
        }
        if (session_idx) output_offset = sizeof(fd_age_gender_res);
        memset((void*)(((u32)dme_agg_out)+output_offset), 0, sizeof(fd_age_gender_res));
        ret = get_fd_age_gender_result((fd_age_gender_res*)(((u32)dme_agg_out)+output_offset), model_config);
    } else {
        // allocate max result (482*4 bytes) len for different DME applications;
        // change DME_OBJECT_MAX if more memory space is needed
        raw_len = sizeof(dme_res);

        if (dme_od_out == 0) {  // allocate raw output result buffer
            dme_od_out = (dme_res *)kmdw_ddr_reserve(raw_len*2);
            dme_out_len = raw_len;  // can't reallocate this
        }
        if (session_idx) output_offset = raw_len;
        struct scpu_to_ncpu_s* comm_out = kdrv_ncpu_get_output();

        if (async_enable[session_idx]) {  // set signal for raw relocation
            if (session_idx) {
                comm_out->output_mem_addr2 = dme_raw_data + 0x100000;
            } else {
                comm_out->output_mem_addr2 = dme_raw_data;
            }
        }
        else
            comm_out->output_mem_addr2 = 0;
        ret = kapp_dme((dme_res*)(((u32)dme_od_out)+output_offset), model_config);
    }
    return ret;
}

uint32_t get_dme_outputs(u16 session_id)
{
    struct kdp_dme_cfg *model_config = p_dme_config_tbl->dme_config;
    uint32_t offset = 0;
    uint32_t output_len = 0;

    ack_idx = session_id & 1;  // find out which output buffer is used
    model_config += active_model_index[ack_idx];
    uint32_t output_num = model_config->output_num;

    if (model_config->img_conf.image_format & IMAGE_FORMAT_RAW_OUTPUT) {
        if(ack_idx) offset = dme_raw_len;
        for (int i = 0; i < output_num; i++){
            int h = *(uint32_t *)(dme_raw_out + offset + (5*i+1)*sizeof(uint32_t));
            int c = *(uint32_t *)(dme_raw_out + offset + (5*i+2)*sizeof(uint32_t));
            int w = *(uint32_t *)(dme_raw_out + offset + (5*i+3)*sizeof(uint32_t));
            output_len = output_len + h * c * round_up(w);
        }
        //output_num+HCWRadixScale+H2C2W2R2S2+FP_DATA1+FP_DATA2
        dme_out_len = (1 + 5*output_num)*sizeof(uint32_t) + output_len;
        output_len = dme_out_len;
    } else if (model_config->img_conf.image_format & IMAGE_FORMAT_MODEL_AGE_GENDER) {
        output_len = sizeof(fd_age_gender_res);
        dme_out_len = output_len;
    } else {
#ifdef CUSTOMER2
        if (model_config->model_id == USER_MODEL_2) {
            output_len = 2 * IMAGENET_TOP_MAX * sizeof(uint32_t);
        }
#else
        if (model_config->model_id == USER_MODEL_2) {
            output_len = (1 + 6*output_num)*sizeof(uint32_t);
        }
#endif
        else if (model_config->model_id == USER_MODEL_0 ||
                    model_config->model_id == USER_MODEL_1) {
            output_len = (1 + 6*output_num)*sizeof(uint32_t);
        } else {
            offset = (u32)dme_od_out;  // find the address of dme_od_out
            if (ack_idx) offset += sizeof(dme_res);
            int det_num = ((dme_res*)(offset))->box_count;
            output_len = 2*sizeof(uint32_t) + det_num * sizeof(struct bounding_box_s);
        }
        dme_out_len = output_len;
    }
    if (output_len > 1000000) {
        err_msg("[get output] BAD output_len of %d\n", output_len);
        output_len = 1000000;  // avoid crash
    }
    return output_len;
}

static int start_dme(u16 session_id, u16 async_mode)
{
    u32 idx = session_id & 1;  // queue size is 2, odd/even

    active_session[idx] = session_id;
    session_status[idx] = 2;  // don't wait for inference thread to update the status
    async_enable[idx] = async_mode;
    if (async_mode) {
    if (idx)
        osThreadFlagsSet(tid_infer, FLAG_INFERENCE_START1);
    else
        osThreadFlagsSet(tid_infer, FLAG_INFERENCE_START2);
    } else {
        do_inference(idx);  // normal mode is blocking
    }
    return 0;
}

static void handle_dme_start(RspPram *rsp, u32 model_size, u32 *fw_info, u16 fw_info_size)
{
    uint32_t *dme_info, *dme_config;
    dlog("model_size [%u], fw_info [0x%x], fw_info_size [%u]", model_size, fw_info, fw_info_size);

    // NOTE: Need to determine if DME structure needs to be set up first before processing images
    info_msg("[kcomm] Start DME mode.\n");
    dme_info = (u32*)KDP_DDR_MODEL_INFO_TEMP;
    memcpy(dme_info, fw_info, fw_info_size);

    uint32_t cmd_addr_offset = 9*sizeof(uint32_t); //cmd addr offset in fw_info
    uint32_t cmd_start_addr = *(uint32_t*)(KDP_DDR_MODEL_INFO_TEMP + cmd_addr_offset);

    // create config table
    int ret = (int) (*dme_info);  // get number of models
    if (p_dme_config_tbl == 0)
        p_dme_config_tbl = (void*)kmdw_ddr_reserve(28*10 + 4);  // reserve space for max 10 models
    memset(p_dme_config_tbl, 0, 284);
    dme_config = (u32*)p_dme_config_tbl;
    *dme_config = ret;  // store number of models
    dme_config++; dme_info++;
    while(ret > 0) {
        *dme_config = *dme_info;  // store model id
        dme_config += 7; dme_info += 14; ret--;
    }
    if (model_size <= 0) {
        rsp->error = MSG_DATA_ERROR;
        rsp->param2 = CMD_DME_START;
        kcomm_send_rsp(CMD_ACK_NACK, NULL, 0);
        return;
    }

    // Load setup, weights and commands
    ret = kcomm_read(cmd_start_addr, model_size); // specify how many bytes

    if (ret == model_size) {
        APP_id = APP_DME;
        rsp->error = NO_ERROR;
        rsp->param2 = ret;
        kcomm_send_rsp(CMD_DME_START, NULL, 0);
    } else {
        rsp->error = MSG_DATA_ERROR;
        rsp->param2 = 0;
        kcomm_send_rsp(CMD_DME_START, NULL, 0);
    }

    kapp_set_dme_mode(true);
}

static void handle_dme_config(RspPram *rsp, u32 *cfg)
{
    dlog("cfg [0x%x]", cfg);
    active_image = 0;  // reinitialize image queue
    int i;
    if (APP_id == APP_DME) {
        struct kdp_dme_cfg *model_config = (struct kdp_dme_cfg*)((u32)p_dme_config_tbl + 4);
        if ((p_dme_config_tbl->model_count == 1) && (*cfg == 1))  // host_lib old version check
            *cfg = model_config->model_id;  // override the model_id
        for (i= 0; i < p_dme_config_tbl->model_count; i++) {
            if (model_config->model_id == *cfg) {
                memcpy(model_config, cfg, sizeof(struct kdp_dme_cfg)-4);
                model_config->img_conf.image_mem_addr = rgb_img_config.image_mem_addr;
                break;
            }
            model_config++;
        }

        if (i >= p_dme_config_tbl->model_count) {  // matching model id not found
            rsp->error = MSG_DME_BAD_MODEL;
            rsp->param2 = 0;
        } else {
            rsp->error = NO_ERROR;
            rsp->param2 = model_config->model_id;
        }
    }
    session_status[0] = 16;  // reinitialize session status for error recovery
    session_status[1] = 16;
    async_enable[0] = 0;
    async_enable[1] = 0;
    kcomm_send_rsp(CMD_DME_CONFIG, NULL, 0);
}

static void handle_dme_send_img(RspPram *rsp, u32 img_size, u16 process_mode, u16 process_model, u32 api_rev)
{
    struct kdp_dme_cfg *dme_conf = p_dme_config_tbl->dme_config;
    u32 image_mem_offset, early_abort = 2;  // two groups of checks for early aborts
    u16 infr_status[2], last_idx, other_idx;
    int idx = 0;
    infr_status[0] = session_status[0];
    infr_status[1] = session_status[1];
    if (APP_id != APP_DME)
        rsp->error = MSG_CONFIG_ERROR;
    else if (active_image > 1)  // check if we have two images already?
        rsp->error = MSG_DME_Q_FULL;  // can't accept another image
    else if (process_mode == 0) { // check if normal mode?
        if ((infr_status[0] != 16) || (infr_status[1] != 16)) {
            err_msg("[Host] Previous ASYNC not completed: [%d] [%d]\n", infr_status[0], infr_status[1]);
            rsp->error = MSG_APP_BAD_MODE;  // no previous async session allowed
        } else early_abort--;
    }
    else early_abort--;
    if (early_abort < 2) {  // start second round of chedks for early aborts
        if (process_model == 0) process_model = dme_conf->model_id;  // use first model as default
        for (idx = 0; idx < p_dme_config_tbl->model_count; idx ++) {
            if ((dme_conf->model_id == process_model) && (dme_conf->output_num)) {
                early_abort = 0;
                rgb_img_config.image_mem_len = dme_conf->img_conf.image_col * dme_conf->img_conf.image_row * 2;
                break;
            }
            dme_conf++;
        }
        if (idx >= p_dme_config_tbl->model_count)
            rsp->error = MSG_DME_BAD_MODEL;
    }

    if (early_abort) {
        rsp->param2 = CMD_DME_SEND_IMAGE;  // add source id
        kcomm_send_rsp(CMD_ACK_NACK, NULL, 0);
        return;
    }

    // find out the current active session
    if (active_session[1] > active_session[0]) last_idx = 1;
    else last_idx = 0;
    other_idx = last_idx ^ 1;
    if ((active_session[last_idx] == 0xffff) && (active_session[other_idx] == 0)) {
        last_idx = other_idx; // take care of roll over case
        other_idx = last_idx ^ 1;
    }
    session_id = active_session[last_idx] + 1;  // assign new session id
    active_image++;
    if (session_id & 1) {
        image_mem_offset = 1280*720*2;
        active_model_index[1] = idx;
    } else {
        image_mem_offset = 0;
        active_model_index[0] = idx;
    }

    if (dme_conf->img_conf.image_format & IMAGE_FORMAT_BYPASS_PRE)
        idx = kcomm_read(DDR_MEM_BASE, img_size);  // specify how many bytes  + image_mem_offset
    else
        idx = kcomm_read(image_pool + image_mem_offset, img_size);
    if (idx != img_size) { // if image not received
        rsp->error = MSG_APP_BAD_IMAGE;
        rsp->param2 = idx;
        *(u32*)rsp->data = 0;
        kcomm_send_rsp(CMD_DME_SEND_IMAGE, rsp->data, 4);
        return;
    }
    dme_conf->img_conf.image_mem_addr = image_pool + image_mem_offset;

    // Allocate DDR memory for output_mem_addr2 once
    if (0 == dme_raw_data)
        dme_raw_data = kmdw_ddr_reserve(0x200000);  // 0x100000 x 2
    
    if (process_mode) { // if asynchronous mode
        start_dme(session_id, 1);  // enable async mode
        rsp->error = NO_ERROR;
        rsp->param2 = process_mode;
        *(u32*)rsp->data = session_id;
        kcomm_send_rsp(CMD_DME_SEND_IMAGE, rsp->data, 4);
        return;
    }

    // normal mode here
    start_dme(session_id, 0);  // don't use async mode

    kcomm_dme_complete(session_id & 1);
    uint32_t output_size = get_dme_outputs(session_id);

    if (output_size == 1000000) {
        rsp->error = MSG_CONFIG_ERROR;
        rsp->param2 = CMD_DME_SEND_IMAGE; // add source id
        kcomm_send_rsp(CMD_ACK_NACK, NULL, 0);
        return;
    }

    rsp->error = NO_ERROR;
    if (api_rev == 1) { // Host Msg protocol v1.6 or earlier ?
        rsp->param2 = output_size;
        kcomm_send_rsp(CMD_DME_SEND_IMAGE, NULL, 0);
    } else {
        rsp->param2 = process_mode;
        *(u32*)rsp->data = output_size;
        kcomm_send_rsp(CMD_DME_SEND_IMAGE, rsp->data, 4);
    }
}

static void handle_dme_get_status(RspPram *rsp, u16 session_request)
{
    dlog("session [%d]", session_id);

    u16 *psession_data = (u16*)&(rsp->param2);
    int idx;

    if (APP_id != 3) { // must be in DME mode
        rsp->error = MSG_CONFIG_ERROR;
        rsp->param2 = 0;
    } else if ((session_request == active_session[0]) || (session_request == active_session[1])){
        if(session_request == active_session[0])
            idx = 0;
        else
            idx = 1;
        if (session_request == completed_session[idx]) {
            rsp->error = 0;
            *psession_data = session_request;
            psession_data++;

            *psession_data = 1;  // output available
            *(u32*)rsp->data = get_dme_outputs(session_request);//session_id

            kcomm_send_rsp(CMD_DME_GET_STATUS, rsp->data, 4);
            return;
        } else {
            if ((completed_session[idx] == 0) && ((active_session[idx] == 255) || (active_session[idx] == 128))) {
                rsp->error = MSG_DME_BAD_SSID;  // initial reset condition
                rsp->param2 = 0;
            } else {
                rsp->error = 0;
                *psession_data = session_request;
                psession_data++;
                *psession_data = 0;  // output not yet available
                DME_loop[idx]++;
            }
        }
    } else {
        rsp->error = MSG_DME_BAD_SSID;
        rsp->param2 = 0;
    }
    kcomm_send_rsp(CMD_DME_GET_STATUS, NULL, 0);
}

static int handle_dme_cmd(MsgHdr *hdr, CmdPram *args, RspPram *rsp, void *data)
{
    u16 cmd = hdr->cmd;
    u32 arg1 = args->param1;

    switch (cmd) {
    /* now use this to get DME result, should be fixed */
    /* case CMD_ACK_NACK:
     *     handle_ack_nack(arg1, arg2);
     *     break; */
    case CMD_DME_START:
        handle_dme_start(rsp, arg1, (u32*)&args->param2, (hdr->msg_len - 4));
        break;
    case CMD_DME_CONFIG:
        handle_dme_config(rsp, (u32*)&args->param1);
        break;
    case CMD_DME_SEND_IMAGE: {
        u16 processing_mode = args->param2;
        u16 model_id = (args->param2) >> 16;
        u32 api_rev = 2;  // default set to Host Msg protocol v1.7
        if(hdr->msg_len < 8) {  // old host_lib api version check
            processing_mode = 0;
            model_id = p_dme_config_tbl->dme_config[0].model_id;  // use first model
            api_rev = 1;  // Host Msg protocol v1.6 or earlier
        }
        handle_dme_send_img(rsp, arg1, processing_mode, model_id, api_rev);
        break;
    }
    case CMD_DME_GET_STATUS:
         handle_dme_get_status(rsp, (u16)arg1);
       break;
    default:
        return -1;
    }

    return 0;
}

static void init_cmd_handler()
{
    init_img_cfg();
    active_session[0] = 255;
    active_session[1] = 128;
    session_status[0] = 16;
    session_status[1] = 16;
}

/* ############################
 * ##    inference thread    ##
 * ############################ */

static void dme_inference_thread(void *argument)
{
    int32_t flags;
    session_status[0] = 16;  // init session status to operation complete
    session_status[1] = 16;

    for (;;) {
        flags = osThreadFlagsWait(FLAG_INFERENCE_START1 | FLAG_INFERENCE_START2, osFlagsWaitAny, osWaitForever);

        if (flags & FLAG_INFERENCE_START1){
            do_inference(1);
            kcomm_dme_complete(1);
        }
        if (flags & FLAG_INFERENCE_START2){
            do_inference(0);
            kcomm_dme_complete(0);
        }
        if (flags == (FLAG_INFERENCE_START1 | FLAG_INFERENCE_START2))
            err_msg("[***]both INFR events present\n");
        if (flags & ~(FLAG_INFERENCE_START1 | FLAG_INFERENCE_START2))
            err_msg("[infr thread] unknown event: %d\n", flags);
    }
}

/* ############################
 * ##    public functions    ##
 * ############################ */
void kcomm_dme_complete(int idx)
{
    idx &= 1;  // limit idx to 0 or 1
    completed_session[idx] = active_session[idx];
    active_image--;
    session_status[idx] = 16;
}

void kcomm_enable_dme_cmds(void)
{
    init_cmd_handler();
    kcomm_reg_cmd_handler(handle_dme_cmd);

    osThreadAttr_t attr;
    memset(&attr, 0, sizeof(attr));
    attr.priority = osPriorityHigh;
    tid_infer = osThreadNew(dme_inference_thread, NULL, &attr);
}
