/*
 * Kneron Tiny Yolo Companion Application
 *
 * Copyright (C) 2020 Kneron, Inc. All rights reserved.
 *
 */

#include <stdio.h>
#include <string.h>
#include "base.h"
#include "kmdw_console.h"
#include "kapp_ops.h"
#include "tiny_yolo_ops.h"
#include "kmdw_model.h"

static int tiny_yolo_start(uint32_t app_id)
{
    if (app_id != APP_TINY_YOLO3)
        return KAPP_ERR;

    return KAPP_OP_OK;
}

static int tiny_yolo_run_image(uint32_t app_id, struct kapp_img_run_s *img_run_p)
{
    int status;

    if (img_run_p == NULL)
        return -1;

    kmdw_model_config_img(&img_run_p->img_cfg, img_run_p->crop_box, img_run_p->pad_values, img_run_p->ext_param);
    kmdw_model_config_result(img_run_p->evt_id, img_run_p->evt_flag);
    status = kmdw_model_run("kapp_tiny_yolo", img_run_p->out, TINY_YOLO_V3, 0);

    if (status == KMDW_MODEL_RUN_RC_ABORT) {
        info_msg("[INFO] Got abort request\n");
        return KAPP_ABORT;
    } else if(status == KMDW_MODEL_RUN_RC_ERROR) {
        return KAPP_ERR;
    } else if(status != IMAGE_STATE_DONE) {
        return KAPP_ERR;
    }

    return KAPP_OP_OK;
}

static int tiny_yolo_get_result_len(uint32_t app_id, void *result_buf)
{
    struct yolo_result_s *out;
    int len;

    out = (struct yolo_result_s *)result_buf;
    len = out->box_count * sizeof(struct bounding_box_s) + sizeof(uint32_t) * 2;
    return len;
}

static struct kapp_ops kapp_tiny_yolo_ops = {
    .start          = tiny_yolo_start,
    .run_image      = tiny_yolo_run_image,
    .get_result_len = tiny_yolo_get_result_len,
};

struct kapp_ops *tiny_yolo_get_ops(void)
{
    return &kapp_tiny_yolo_ops;
}
