/*
 * Kneron Tiny Yolo Application
 *
 * Copyright (C) 2020 Kneron, Inc. All rights reserved.
 *
 */

#include <cmsis_os2.h>
#include <string.h>
#include <stdlib.h>
#include "utility.h"
#include "kmdw_console.h"
#include "power_manager.h"
#include "kapp_tiny_yolo_inf.h"

typedef void  (*console_cmd_func)(void);

struct console_cmd_info
{
    char *desc;
    console_cmd_func func;
};

static osThreadId_t tid_console;
static osThreadId_t tid_inference;

static int started;

static void kapp_tiny_yolo_start(void)
{
    if (!started) {
        osThreadFlagsSet(tid_inference, FLAGS_YOLO_START_EVT);
        started = !started;
    }
}

static void kapp_tiny_yolo_stop(void)
{
    if (started) {
        osThreadFlagsSet(tid_inference, FLAGS_YOLO_STOP_EVT);
        started = !started;
    }
}

static void kapp_tiny_yolo_toggle(void)
{
    osThreadFlagsSet(tid_inference, FLAGS_YOLO_TOGGLE_EVT);
}

static void kapp_quit(void)
{
    DSG("Bye bye !!");
    power_manager_shutdown();
}

struct console_cmd_info app_idle_mode_array[] = {
    {"Start Tiny Yolo",             kapp_tiny_yolo_start},
    {"Stop Tiny Yolo",              kapp_tiny_yolo_stop},
    {"Toggle pipeline mode",        kapp_tiny_yolo_toggle},
    {"Quit",                        kapp_quit},
};

static void power_btn_handler(int released)
{
    if (released) {
        if (started)
            kapp_tiny_yolo_stop();
        else
            kapp_tiny_yolo_start();
    }
}

void console_thread(void *arg)
{
    int id = 0;
    int i = 0;
    uint32_t idle_cmd_size = ARRAY_SIZE(app_idle_mode_array);
    char buf[64];
    console_cmd_func cmd_func;

    tid_inference = kapp_tiny_yolo_inf_init();
    if (tid_inference == NULL) {
        err_msg("[ERROR] Fail to create Tiny Yolo threads");
        return;
    }

    power_button_register(power_btn_handler);
    
    while(1)
    {
        if (id)
            goto cmd_prompt;

        DSG("\n === Menu === ");
        for (i = 0; i < idle_cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_idle_mode_array[i].desc);
            DSG("%s", buf);
        }

cmd_prompt:
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));

        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id <= idle_cmd_size)
        {
            cmd_func = app_idle_mode_array[id - 1].func;
            if (cmd_func) {
                cmd_func();    
            }
        }
        else {
            if (id)
                err_msg("Invalid command: %d\n", id);
            continue;
        }
    }
}

void console_entry(void)
{
    tid_console = osThreadNew(console_thread, NULL, NULL);
    osThreadSetPriority (tid_console, osPriorityBelowNormal);
}

