/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    main.c
 *      Purpose: RTX for Kneron
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "kdrv_uart.h"

#include "kneron_mozart.h"                // Device header
#include "kneron_mozart_ext.h"
#include "cmsis_os2.h"                    // ARM::CMSIS:RTOS2:Keil RTX5
#include "base.h"
#include "io.h"
#include "string.h"
#include "kdrv_ddr.h"

#ifdef KL520
#include "kdrv_scu_ext.h"
#include "kdrv_power.h"
#include "kdrv_system.h"
#include "kdrv_clock.h"
#include "power_manager.h"
#include "kdrv_scu.h"
#endif

#include "kmdw_console.h"


extern uint32_t func_code_in_ddr(uint32_t a, uint32_t b);

/**
 * @brief main, main dispatch function
 */
int main(void) 
{
	SystemCoreClockUpdate();            // System Initialization 
    osKernelInitialize();               // Initialize CMSIS-RTOS


    /* below is some primiary system init settings */
    kdrv_system_init();	// primary system init
    kdrv_system_init_ncpu(); // this is a must for ddr
    kdrv_ddr_system_init(DDR_INIT_ALL); // enable ddr


    /* below is some middleware init settings */
    kdrv_uart_initialize(); // for log printing
    kmdw_console_init(UART0_DEV);   // enable console mdw for printing

    // below code of function will be executed from DDR
    uint32_t var = func_code_in_ddr(123, 456);

    DSG("func_code_in_ddr() is working, it returns %d", var);


    //application is triggered in host_com.c
    if (osKernelGetState() == osKernelReady) {
        osKernelStart();
    }

    while(1) {
    }
}
