/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    main.c
 *      Purpose: RTX for Kneron
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "kneron_mozart.h"                // Device header
#include "kneron_mozart_ext.h"
#include "cmsis_os2.h"                    // ARM::CMSIS:RTOS2:Keil RTX5
#include "Driver_Common.h"
#include "base.h"
#include "io.h"
#include "string.h"
#include "kdrv_ddr.h"
#include "kmdw_model.h"
#ifdef KL520
#include "kdrv_scu_ext.h"
#include "bootloader.h"
#include "kdrv_power.h"
#include "kdrv_system.h"
#include "kdrv_clock.h"
#include "power_manager.h"
#include "kdrv_scu.h"
#endif
#include "kmdw_memory.h"     //for ddr_malloc
#include "kdp_ddr_table.h"  //for default ddr table
#include "kmdw_console.h"
#include "console_example.h"
#include "kdrv_uart.h"
#include "pinmux.h"

/**
 * @brief main, main dispatch function
 */
int main(void) 
{
    SystemCoreClockUpdate();            // System Initialization
    osKernelInitialize();               // Initialize CMSIS-RTOS
    
    pinmux_init();
    kdrv_uart_initialize();
	kmdw_console_init(UART0_DEV);		// enable console mdw for printing
    kmdw_model_init();

    console_example_entry();

    //application is triggered in host_com.c
    if (osKernelGetState() == osKernelReady) {
        osKernelStart();
    }
    
    while(1) {
    }
}


