#include <stdio.h>
#include <stdlib.h>
#include "libusb.h"

int main(int argc, char *argv[])
{
	int status;

	if (0 != libusb_init(NULL))
	{
		printf("libusb_init() failed\n");
		return NULL;
	}

	libusb_device_handle *devH = libusb_open_device_with_vid_pid(NULL, 0x3231, 0x0100);
	if (!devH)
	{
		printf("libusb open failed\n");
		return NULL;
	}

	status = libusb_set_configuration(devH, 1);
	if (status)
	{
		printf("libusb_set_configuration() failed: %s\n", libusb_strerror((enum libusb_error)status));
		return NULL;
	}

	status = libusb_claim_interface(devH, 0);
	if (status)
	{
		printf("libusb_claim_interface() failed: %s\n", libusb_strerror((enum libusb_error)status));
		return NULL;
	}

	printf("kneron usb device open OK\n");

	int buf_len = 4 * 1024 * 1024;
	unsigned char *buf = (unsigned char *)malloc(buf_len);

	static unsigned int out_count = 0;
	static unsigned int in_count = 0;

	srand(time(NULL));

	//for(int i=0;i<4;i++)
	while(1)
	{
		int length = (rand() % buf_len) + 1;
		int transferred;

#if 1
		printf("send %d bytes\n", length);
		status = libusb_bulk_transfer(devH, 0x1, buf, length, &transferred, 100000);
		if (status != 0)
		{
			printf("bulk out transfer failed error: %d\n", status);
			break;
		}

		if (length != transferred)
		{
			printf("bulk out transfer size not match %d vs %d\n", length, transferred);
			break;
		}

		if ((length % 512) == 0)
		{
			status = libusb_bulk_transfer(devH, 0x1, NULL, 0, &transferred, 100000);
			if (status != 0)
			{
				printf("bulk out ZLP failed: %d\n", status);
				break;
			}
		}

		out_count++;
		printf("%d: bulk out %d bytes done\n", out_count, transferred);

#endif

#if 1
		status = libusb_bulk_transfer(devH, 0x82, buf, buf_len, &transferred, 1000000);
		if (status != 0)
		{
			printf("bulk in transfer failed error: %d\n", status);
			break;
		}

		in_count++;
		printf("%d: bulk in %d bytes done\n", in_count, transferred);
#endif

		//Sleep(10);

	}
	

	free(buf);

	libusb_exit(NULL);

	return 0;
}