/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    main.c
 *      Purpose: RTX for Kneron
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include "cmsis_os2.h" // ARM::CMSIS:RTOS2:Keil RTX5
#include "kdrv_ddr.h"
#include "kdrv_system.h"
#include "kmdw_memory.h"    //for ddr_malloc
#include "kdp_ddr_table.h" //for default ddr table
#include "kmdw_console.h"

extern void usbd_sync_example_init(void);

/**
 * @brief main, main dispatch function
 */
int main(void)
{
    SystemCoreClockUpdate(); // System Initialization
    osKernelInitialize();    // Initialize CMSIS-RTOS

    /* below is some primiary system init settings */
    kdrv_system_init();                 // primary system init
    kdrv_system_init_ncpu();            // this is a must for ddr
    kdrv_ddr_system_init(DDR_INIT_ALL); // enable ddr

    /* below is some middleware init settings */
    kmdw_ddr_init(KDP_DDR_HEAP_HEAD_FOR_MALLOC + 1, KDP_DDR_MODEL_RESERVED_END); // for memory alloc from ddr
    kdrv_uart_initialize();                                                            // for log printing
    kmdw_console_init(UART0_DEV);                                               // enable console mdw for printing

    /* init the application */
    usbd_sync_example_init();

    if (osKernelGetState() == osKernelReady)
    {
        osKernelStart();
    }

    while (1)
    {
    }
}
