#include "sample_app_wifi.h"
#include <string.h>
#include <stdlib.h>
#include <cmsis_os2.h>
#include "utility.h"
#include "kmdw_console.h"
#include "kmdw_memxfer.h"
#include "kmdw_display.h"
#include "kdrv_uart.h"
#include "kdrv_system.h"
#include "kdrv_power.h"
#include "kdrv_ddr.h"
#include "rtc.h"
#include "kdrv_clock.h"
#include "power_manager.h"
#include "kapp.h"
#include "kdp_ddr_table.h"
#include "kmdw_memory.h"
#include "kl520_api.h"
#include "kl520_api_camera.h"
#include "kl520_api_fdfr.h"
#include "kl520_api_device_id.h"
#include "media/kdp_inference.h"

typedef int (*console_cmd_func)(void);

#include "kdev_wifi.h"
#include "WiFi_ESP8266.h"

//uint8_t ip_server_addr[] = {192,168,20,27};
//uint32_t ip_len = 4; 
//uint16_t server_port = 2002;

uint8_t kapp_tcp_status = 0x00;

socket_info kapp_socket ; 
#define TCP_SOCKET_TEST 1

static ARM_WIFI_CONFIG_t kdev_wifi_config =
{
	.ssid     = WIFI_STA_SSID,
	.pass     = WIFI_STA_PASS,
	.security = WIFI_STA_SECURITY,
	.ch       = WIFI_STA_CH,
};

void kapp_wifi_init(void)
{
	uint8_t bssid_buf[7] = {0};
	uint32_t bssid_len = 7;
	int32_t ret = 0;
	
	ret = kdev_WIFI_Initialize_Uninitialize(true);
	
	kdev_WIFI_DV_Initialize();
	
	
	
	ret = kdev_WIFI_GetOption_BSSID(0u,bssid_buf,&bssid_len);
	if(ret < 0){
		DSG("wifi get bssid error :%d,%d",ret,bssid_len);
	}
}


int32_t station_init(ARM_WIFI_CONFIG_t *config)
{
		int32_t ret = 0;
    ret = kdev_WIFI_Activate_Deactivate(WIFI_INTERFACE_STATION,config,true);
		if(ret != ARM_DRIVER_OK)
		{
			DSG("wifi station init activate error:%d",ret);
		}
		return ret;
}

void kapp_wifi_tcpsocket_init(socket_info* test_socket)
{
	test_socket->af = ARM_SOCKET_AF_INET;
	test_socket->protocol = ARM_SOCKET_IPPROTO_TCP;
	test_socket->type = ARM_SOCKET_SOCK_STREAM;
}

void kapp_wifi_udpsocket_init(socket_info* test_socket)
{
	test_socket->af = ARM_SOCKET_AF_INET;
	test_socket->protocol = ARM_SOCKET_IPPROTO_UDP;
	test_socket->type = ARM_SOCKET_SOCK_DGRAM;
}
 
int32_t kapp_wifi_client_socket_creat(socket_info* test_socket)
{
	int32_t ret = 0;
	
	ret = station_init(&kdev_wifi_config);
	if(ret != ARM_DRIVER_OK)
	{
		DSG("station init failed ret:%d",ret);
		return ret;
	}
	else
	{
		osDelay(5000);
	}

	test_socket->sock = kdev_socket_creat(test_socket->af,test_socket->protocol,test_socket->type );
	if(test_socket->sock < 0)
	{
		DSG("station socket creat failed ret:%d",ret);
		return -1;
	}
	
	return test_socket->sock;
}
	
int32_t kapp_wifi_tcp_client_socket_connect(int32_t s)
{
	int32_t ret = 0;
	uint8_t ip_socket_server[4] = {192,168,20,27};
	uint16_t server_port = 2002;
	
	//sscanf(WIFI_SOCKET_SERVER_IP, "%hhu.%hhu.%hhu.%hhu", &ip_socket_server[0], &ip_socket_server[1], &ip_socket_server[2], &ip_socket_server[3]);
	ret = kdev_socket_connect(s, ip_socket_server,0x04, server_port, 0);
	if(ret < 0)
	{
		DSG("station socket connect failed ret:%d",ret);
	}
	else
	{
		kapp_tcp_status = 1;
	}
	
	return ret;
}


struct console_cmd_info
{
    char *desc;
    console_cmd_func func;
};

static u32 m_curr_user_idx = 0;
static int reg_index;
osThreadId_t tid_console_example = 0;
#define RDD_MODE_FLAG 0
struct console_cmd_info app_idle_mode_array[] = {
//    {"Register mode",               sample_app_register_mode  },
//    {"Add mode",                    sample_app_add_mode  },
    {"Inference mode",              sample_app_inference_mode  },
//    {"Delete mode",                 sample_app_delete_mode  },
    {"System on/off test",          sample_app_system_test_mode },
    {"Quit",                        sample_app_quit  },
};

struct console_cmd_info app_register_mode_array[] = {
	#if RDD_MODE_FLAG
    {"Manual register user face",   sample_face_register  },
    {"Auto register user face",     sample_face_auto_register  },
    {"Abort register record",       sample_face_cancel_register  },
    {"Back to idle mode",           sample_app_back_idle  },
	#endif
};

struct console_cmd_info app_add_mode_array[] = {
	#if RDD_MODE_FLAG
    {"Add user face",               sample_face_add  },
    {"Back to idle mode",           sample_app_back_idle  },
	#endif 
};

struct console_cmd_info app_inference_mode_array[] = {
    {"Inference user face",             sample_face_recognition  },
    {"Start loop inference user face",  sample_face_recognition_loop  },
    {"Stop loop inference user face",   sample_face_recognition_loop_stop  },
    {"Back to idle mode",               sample_app_back_idle  },
};

struct console_cmd_info app_delete_mode_array[] = {
	#if RDD_MODE_FLAG
    {"Delete user face",            sample_face_del_user  },
    {"Delete all user faces",       sample_face_del_all  },    
    {"Back to idle mode",           sample_app_back_idle  },
	#endif
};

struct console_cmd_info app_test_mode_array[] = {
    {"System test on/off",          sample_system_test  },   
    {"Back to idle mode",           sample_app_back_idle  },
};

int sample_prepare_inference(void)
{
    if(kl520_api_camera_is_running(CAMERA_RGB_IDX) == FALSE) {
        kl520_api_camera_open(CAMERA_RGB_IDX);
        kl520_api_camera_start(CAMERA_RGB_IDX);
    }
    if(kl520_api_dp_init_get() == FALSE) {
        kl520_api_dp_open(CAMERA_RGB_IDX, LCD_WIDTH, LCD_HEIGHT, V2K_PIX_FMT_RGB565);
    }
    if(kl520_api_camera_is_running(CAMERA_NIR_IDX) == FALSE) {
        kl520_api_camera_open(CAMERA_NIR_IDX);
        kl520_api_camera_start(CAMERA_NIR_IDX);
    }
    kl520_api_fdfr_model_init();
    return 0;
}

int sample_face_recognition(void)
{
    int ret;
    kl520_api_set_inference_img_src(KDP_INFERENCE_SINGLE_IMGSRC);
    kl520_api_face_recognition_set_timeout(APP_INFERENCE_TIMEOUT);
    ret = kl520_api_face_recognition();
    info_msg("%s - ret:%d\n", __func__, ret);
    return ret;
}

int sample_face_recognition_loop(void)
{
    int ret;
    kl520_api_set_inference_img_src(KDP_INFERENCE_SINGLE_IMGSRC);
    ret = kl520_api_face_recognition_loop();
    info_msg("%s - ret:%d\n", __func__, ret);
    return ret;
}

int sample_face_recognition_loop_stop(void)
{
    int ret;
    ret = kl520_api_face_recognition_loop_stop();
    info_msg("%s - ret:%d\n", __func__, ret);
    return ret;
}

int sample_face_cancel_register(void)
{
    int ret;
    ret = kl520_api_face_cancel_register();
    dbg_msg("%s - ret:0x%x\n", __func__, ret);
    return ret;
}

int sample_face_register(void)
{
    int ret;
    ret = kl520_api_face_register(m_curr_user_idx);
    dbg_msg("%s - ret:0x%x\n", __func__, ret);
    return ret;
}

int sample_face_auto_register(void)
{
    int ret;
    ret = kl520_api_face_auto_register(m_curr_user_idx, reg_index);
    dbg_msg("%s m_curr_user_idx:%d, reg_index:%d - ret:0x%x\n", __func__, m_curr_user_idx, reg_index, ret);
    return ret;
}

int sample_face_add(void)
{
    int ret;
    ret = kl520_api_face_add(m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
    return ret;
}

int sample_face_del_user(void)
{
    int ret;
    ret = kl520_api_face_del(FALSE, m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
    return ret;
}

int sample_face_del_all(void)
{
    int ret;
    ret = kl520_api_face_del(TRUE, m_curr_user_idx);
    dbg_msg("%s m_curr_user_idx:%d - ret:0x%x\n", __func__, m_curr_user_idx, ret);
    return ret;
}

int sample_system_test(void)
{
    char buf[64];
    int count, total;
    int randtime;

l_start:
    DSG("\n *************************************************");
    DSG(" ** This system on/off test flow shows as below **");
    DSG(" ** 1. Open cameras/lcdc and run inference      **");
    DSG(" ** 2. Rand 1~5 seconds to stop inference       **");
    DSG(" ** 3. Rand 1~5 seconds close camera/lcdc       **");
    DSG(" ** 4. Rand 1~5 seconds repeat step 1           **");
    DSG(" *************************************************");
    DSG_NOLF("Test count >> ");
    kmdw_console_echo_gets(buf, sizeof(buf));
    count = 0;
    total = atoi(strtok(buf, " \r\n\t"));
    if(total < 0) {
        DSG("Wrong input value:%d", total);
        goto l_start;
    }
    while(count != total) {
        kl520_api_face_recognition_loop();
        randtime = (rand()%5) + 1;
        osDelay(randtime*1000);
        kl520_api_face_recognition_loop_stop();
        randtime = (rand()%5) + 1;
        osDelay(randtime*1000);
        kl520_api_face_close();
        randtime = (rand()%5) + 1;
        osDelay(randtime*1000);
        ++count;
        DSG("===================================================== %d PASS", count);
    }
    sample_prepare_inference();
    return 0;
}

int sample_app_register_mode(void)
{
    int id;
    int i = 0;
    uint32_t cmd_size = ARRAY_SIZE(app_register_mode_array);
    char buf[64];
    console_cmd_func cmd_func;

    while(1)
    {
        int user_idx;
        int remaining_fmap_cnt;
        
        DSG("\n === Register Mode === ");
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_register_mode_array[i].desc);
            DSG("%s", buf);
        }
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));
        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id < cmd_size)
        {
            cmd_func = app_register_mode_array[id - 1].func;
            if (cmd_func == sample_face_register) {
                user_idx = kdp_fdfr_get_available_db_idx();
                if(user_idx >= MAX_USER)
                {
                    DSG_NOLF(" No DB space anymore, DB Maximum is %d\n", MAX_USER);
                    continue;
                }
l_rgst_cmd:
                remaining_fmap_cnt = kdp_fdfr_get_remaining_fmap_cnt(user_idx);
                DSG_NOLF(" try to register index %d, remaining fmap_cnt %d\n", user_idx, remaining_fmap_cnt);
                if (kapp_db_get_fm0_cnt(user_idx)==65535) {
                    DSG_NOLF(" try to register index %d fm cnt is uninitialized\n", user_idx);
                }
                else if(remaining_fmap_cnt > 0)
				{
                    DSG_NOLF(" Assign UID=%d, ready to capture image(%d) to register, Y/N? ",
						user_idx, remaining_fmap_cnt);
                }
                else if(remaining_fmap_cnt == 0)
                {
                    DSG_NOLF(" UID=%d, remaining %d can be registered\n",
						user_idx, remaining_fmap_cnt);
                    continue;
                }
                kmdw_console_echo_gets(buf, sizeof(buf));

                if(strncmp(buf, "Y", 1) == 0 || strncmp(buf, "y", 1) == 0)
                    m_curr_user_idx = user_idx;
                else if(strncmp(buf, "N", 1) == 0 || strncmp(buf, "n", 1) == 0)
                    continue;
                else
                    goto l_rgst_cmd;

                cmd_func();
                goto l_rgst_cmd;
            }
            else if (cmd_func == sample_face_auto_register) {
                kl520_api_face_recognition_loop_stop();
                user_idx = kdp_fdfr_get_available_db_idx();
                if(user_idx >= MAX_USER)
                {
                    DSG_NOLF(" No DB space anymore, DB Maximum is %d\n", MAX_USER);
                    continue;
                }
                m_curr_user_idx = user_idx;
                DSG_NOLF(" Start Auto Register for UID:%d! (Move Your Face Slowly)\n", user_idx);
                reg_index = 0;
                for (int i = 0; i < 20; i++) {
                    //DSG_NOLF(" reg_index %d !\n", reg_index);
                    info_msg(".");
                    cmd_func();
                    reg_index++;
                    kdrv_delay_us(50000); //adjust according to UX
                }
                info_msg(" Done \n");
                kl520_api_face_register_cfm(m_curr_user_idx);                                      
            }
            else if (cmd_func == sample_face_cancel_register) {
                cmd_func();
            }
        }
        else if (cmd_size == id) {
            cmd_func = app_register_mode_array[id - 1].func;
            break;
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
    return 0;
}

int sample_app_add_mode(void)
{
    int id;
    int i = 0;
    uint32_t cmd_size = ARRAY_SIZE(app_add_mode_array);
    char buf[64];
    console_cmd_func cmd_func;

    while(1)
    {        
        DSG("\n === Add Mode === ");
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_add_mode_array[i].desc);
            DSG("%s", buf);
        }
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));
        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id < cmd_size)
        {
            cmd_func = app_add_mode_array[id - 1].func;
            cmd_func();
        }
        else if (cmd_size == id) {
            cmd_func = app_add_mode_array[id - 1].func;
            break;
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
    return 0;
}

int sample_app_inference_mode(void)
{
    int id;
    int i = 0;
    uint32_t cmd_size = ARRAY_SIZE(app_inference_mode_array);
    char buf[64];
    console_cmd_func cmd_func;

    while(1)
    {        
        DSG("\n === Inference Mode ===");
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_inference_mode_array[i].desc);
            cmd_func = app_inference_mode_array[i].func;
            if(cmd_func == sample_face_recognition) {
                sprintf(buf, "(%2d) %s - failure timeout %d seconds", i + 1, app_inference_mode_array[i].desc, APP_INFERENCE_TIMEOUT);
            }
            else {
                sprintf(buf, "(%2d) %s", i + 1, app_inference_mode_array[i].desc);
            }
            DSG("%s", buf);
        }
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));
        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id < cmd_size)
        {
            cmd_func = app_inference_mode_array[id - 1].func;
            cmd_func();
        }
        else if (cmd_size == id) {
            cmd_func = app_inference_mode_array[id - 1].func;
            break;
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
    return 0;
}

int sample_app_delete_mode(void)
{
    int id;
    int i = 0;
    uint32_t cmd_size = ARRAY_SIZE(app_delete_mode_array);
    char buf[64];
    console_cmd_func cmd_func;

    while(1)
    {        
        DSG("\n === Delete Mode === ");
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_delete_mode_array[i].desc);
            DSG("%s", buf);
        }
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));
        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id < cmd_size)
        {
            cmd_func = app_delete_mode_array[id - 1].func;
            if(cmd_func == sample_face_del_user) {
                DSG_NOLF(" [del_user] Please enter 0 ~ %d : ", (MAX_DB_USERS - 1));
                kmdw_console_echo_gets(buf, sizeof(buf));
                m_curr_user_idx = atoi(strtok(buf, " \r\n\t"));
                if(m_curr_user_idx == 0 && buf[0] != '0')  {
                    DSG(" [del_user] wrong input, abort !");
                    continue;
                }
                cmd_func();
            }
            else if(cmd_func == sample_face_del_all) {
                cmd_func();
            }
        }
        else if (cmd_size == id) {
            cmd_func = app_delete_mode_array[id - 1].func;
            break;
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
    return 0;
}

int sample_app_system_test_mode(void)
{
    int id;
    int i = 0;
    uint32_t cmd_size = ARRAY_SIZE(app_test_mode_array);
    char buf[64];
    console_cmd_func cmd_func;
    while(1)
    {        
        DSG("\n === System On/Off Test Mode === ");
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_test_mode_array[i].desc);
            DSG("%s", buf);
        }
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));
        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id < cmd_size)
        {
            cmd_func = app_test_mode_array[id - 1].func;
            cmd_func();
        }
        else if (cmd_size == id) {
            cmd_func = app_test_mode_array[id - 1].func;
            break;
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
    return 0;
}

int sample_app_back_idle(void)
{
    return 0;
}

int sample_app_quit(void)
{
    DSG("Bye bye !!");
    power_manager_shutdown();
    return 0;
}




void app_console_thread(void *arg)
{
    int id;
    int i = 0;
    uint32_t idle_cmd_size = ARRAY_SIZE(app_idle_mode_array);
    char buf[64];
    console_cmd_func cmd_func;
		
		int32_t ret = 0;

    sample_prepare_inference();
	
		#if 1
		kapp_wifi_init();
		#if TCP_SOCKET_TEST
		kapp_wifi_tcpsocket_init(&kapp_socket);
		ret = kapp_wifi_client_socket_creat(&kapp_socket);
		if( ret < 0)
		{
			err_msg("client socket creat faile: %d", ret);
		}
		else
		{
			ret = kapp_wifi_tcp_client_socket_connect(kapp_socket.sock);
			if(ret < 0)
			{
				err_msg("client socket connect faile: %d", ret);
			}
		}
		#else
		kapp_wifi_udpsocket_init(&kapp_socket);
		#endif
		#endif
		
		
    while(1)
    {
        id = 0;
        DSG("\n === IDLE Mode === ");
        for (i = 0; i < idle_cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s", i + 1, app_idle_mode_array[i].desc);
            DSG("%s", buf);
        }
        DSG_NOLF(" command >> ");
        kmdw_console_echo_gets(buf, sizeof(buf));

        id = atoi(strtok(buf, " \r\n\t"));
        if (id > 0 && id <= idle_cmd_size)
        {
            cmd_func = app_idle_mode_array[id - 1].func;
            if (cmd_func) {
                cmd_func();    
            }
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
}



void sample_doorlock_entry(void)
{
    /*Configure camera setting*/
#if (IMGSRC_IN_0)
    kl520_api_camera_setting(0, IMGSRC_0_WIDTH, IMGSRC_0_HEIGHT, IMGSRC_0_FORMAT);
#endif
#if (IMGSRC_IN_1)
    kl520_api_camera_setting(1, IMGSRC_1_WIDTH, IMGSRC_1_HEIGHT, IMGSRC_1_FORMAT);
#endif
    if(0 != kdp_fdfr_inference_thread_create()) {
        err_msg("[ERROR] Fail to creahte kdp_fdfr_inference_thread");
        return;
    }

    if(kdp_fdfr_draw_thread_create() != 0) {
        err_msg("[ERROR] Fail to create kdp_fdfr_draw_thread_create");
        return;
    }
    tid_console_example = osThreadNew(app_console_thread, NULL, NULL);
    osThreadSetPriority (tid_console_example, osPriorityBelowNormal);
}



